@@ -12,6 +12,7 @@
 
 namespace Composer\Command;
 
+use Composer\Composer;
 use Symfony\Component\Console\Input\InputInterface;
 use Symfony\Component\Console\Output\OutputInterface;
 
@@ -20,11 +21,14 @@ use Symfony\Component\Console\Output\OutputInterface;
  */
 class AboutCommand extends BaseCommand
 {
+    /**
+     * @return void
+     */
     protected function configure()
     {
         $this
             ->setName('about')
-            ->setDescription('Shows the short information about Composer.')
+            ->setDescription('Shows a short information about Composer.')
             ->setHelp(
                 <<<EOT
 <info>php composer.phar about</info>
@@ -35,9 +39,11 @@ EOT
 
     protected function execute(InputInterface $input, OutputInterface $output)
     {
+        $composerVersion = Composer::getVersion();
+
         $this->getIO()->write(
             <<<EOT
-<info>Composer - Dependency Manager for PHP</info>
+<info>Composer - Dependency Manager for PHP - version $composerVersion</info>
 <comment>Composer is a dependency manager tracking local dependencies of your projects and libraries.
 See https://getcomposer.org/ for more information.</comment>
 EOT
