@@ -16,6 +16,8 @@ use Composer\Factory;
 use Composer\IO\IOInterface;
 use Composer\Config;
 use Composer\Composer;
+use Composer\Package\BasePackage;
+use Composer\Package\CompletePackageInterface;
 use Composer\Repository\CompositeRepository;
 use Composer\Repository\RepositoryFactory;
 use Composer\Script\ScriptEvents;
@@ -36,6 +38,9 @@ use Symfony\Component\Console\Output\OutputInterface;
  */
 class ArchiveCommand extends BaseCommand
 {
+    /**
+     * @return void
+     */
     protected function configure()
     {
         $this
@@ -44,11 +49,11 @@ class ArchiveCommand extends BaseCommand
             ->setDefinition(array(
                 new InputArgument('package', InputArgument::OPTIONAL, 'The package to archive instead of the current project'),
                 new InputArgument('version', InputArgument::OPTIONAL, 'A version constraint to find the package to archive'),
-                new InputOption('format', 'f', InputOption::VALUE_REQUIRED, 'Format of the resulting archive: tar or zip'),
+                new InputOption('format', 'f', InputOption::VALUE_REQUIRED, 'Format of the resulting archive: tar, tar.gz, tar.bz2 or zip (default tar)'),
                 new InputOption('dir', null, InputOption::VALUE_REQUIRED, 'Write the archive to this directory'),
                 new InputOption('file', null, InputOption::VALUE_REQUIRED, 'Write the archive with the given file name.'
                     .' Note that the format will be appended.'),
-                new InputOption('ignore-filters', false, InputOption::VALUE_NONE, 'Ignore filters when saving package'),
+                new InputOption('ignore-filters', null, InputOption::VALUE_NONE, 'Ignore filters when saving package'),
             ))
             ->setHelp(
                 <<<EOT
@@ -107,6 +112,17 @@ EOT
         return $returnCode;
     }
 
+    /**
+     * @param string|null $packageName
+     * @param string|null $version
+     * @param string $format
+     * @param string $dest
+     * @param string|null $fileName
+     * @param bool $ignoreFilters
+     *
+     * @return int
+     * @throws \Exception
+     */
     protected function archive(IOInterface $io, Config $config, $packageName = null, $version = null, $format = 'tar', $dest = '.', $fileName = null, $ignoreFilters = false, Composer $composer = null)
     {
         if ($composer) {
@@ -140,6 +156,12 @@ EOT
         return 0;
     }
 
+    /**
+     * @param string      $packageName
+     * @param string|null $version
+     *
+     * @return (BasePackage&CompletePackageInterface)|false
+     */
     protected function selectPackage(IOInterface $io, $packageName, $version = null)
     {
         $io->writeError('<info>Searching for the specified package.</info>');
@@ -171,6 +193,10 @@ EOT
             return false;
         }
 
+        if (!$package instanceof CompletePackageInterface) {
+            throw new \LogicException('Expected a CompletePackageInterface instance but found '.get_class($package));
+        }
+
         return $package;
     }
 }
