@@ -15,6 +15,7 @@ namespace Composer;
 use Composer\Config\ConfigSourceInterface;
 use Composer\Downloader\TransportException;
 use Composer\IO\IOInterface;
+use Composer\Pcre\Preg;
 use Composer\Util\Platform;
 use Composer\Util\ProcessExecutor;
 
@@ -23,14 +24,22 @@ use Composer\Util\ProcessExecutor;
  */
 class Config
 {
+    const SOURCE_DEFAULT = 'default';
+    const SOURCE_COMMAND = 'command';
+    const SOURCE_UNKNOWN = 'unknown';
+
     const RELATIVE_PATHS = 1;
 
+    /** @var array<string, mixed> */
     public static $defaultConfig = array(
         'process-timeout' => 300,
         'use-include-path' => false,
-        'preferred-install' => 'auto',
+        'allow-plugins' => null, // null for BC for now, will become array() after July 2022
+        'use-parent-dir' => 'prompt',
+        'preferred-install' => 'dist',
         'notify-on-install' => true,
         'github-protocols' => array('https', 'ssh', 'git'),
+        'gitlab-protocol' => null,
         'vendor-dir' => 'vendor',
         'bin-dir' => '{$vendor-dir}/bin',
         'cache-dir' => '{$home}/cache',
@@ -54,6 +63,7 @@ class Config
         'bitbucket-expose-hostname' => true,
         'disable-tls' => false,
         'secure-http' => true,
+        'secure-svn-domains' => array(),
         'cafile' => null,
         'capath' => null,
         'github-expose-hostname' => true,
@@ -75,23 +85,30 @@ class Config
         // bearer
     );
 
+    /** @var array<string, mixed> */
     public static $defaultRepositories = array(
         'packagist.org' => array(
             'type' => 'composer',
-            'url' => 'https?://repo.packagist.org',
-            'allow_ssl_downgrade' => true,
+            'url' => 'https://repo.packagist.org',
         ),
     );
 
+    /** @var array<string, mixed> */
     private $config;
+    /** @var ?string */
     private $baseDir;
+    /** @var array<int|string, mixed> */
     private $repositories;
     /** @var ConfigSourceInterface */
     private $configSource;
     /** @var ConfigSourceInterface */
     private $authConfigSource;
+    /** @var bool */
     private $useEnvironment;
+    /** @var array<string, true> */
     private $warnedHosts = array();
+    /** @var array<string, string> */
+    private $sourceOfConfigValue = array();
 
     /**
      * @param bool   $useEnvironment Use COMPOSER_ environment variables to replace config settings
@@ -101,26 +118,52 @@ class Config
     {
         // load defaults
         $this->config = static::$defaultConfig;
+
+        // TODO after July 2022 remove this and update the default value above in self::$defaultConfig + remove note from 06-config.md
+        if (strtotime('2022-07-01') < time()) {
+            $this->config['allow-plugins'] = array();
+        }
+
         $this->repositories = static::$defaultRepositories;
         $this->useEnvironment = (bool) $useEnvironment;
         $this->baseDir = $baseDir;
+
+        foreach ($this->config as $configKey => $configValue) {
+            $this->setSourceOfConfigValue($configValue, $configKey, self::SOURCE_DEFAULT);
+        }
+
+        foreach ($this->repositories as $configKey => $configValue) {
+            $this->setSourceOfConfigValue($configValue, 'repositories.' . $configKey, self::SOURCE_DEFAULT);
+        }
     }
 
+    /**
+     * @return void
+     */
     public function setConfigSource(ConfigSourceInterface $source)
     {
         $this->configSource = $source;
     }
 
+    /**
+     * @return ConfigSourceInterface
+     */
     public function getConfigSource()
     {
         return $this->configSource;
     }
 
+    /**
+     * @return void
+     */
     public function setAuthConfigSource(ConfigSourceInterface $source)
     {
         $this->authConfigSource = $source;
     }
 
+    /**
+     * @return ConfigSourceInterface
+     */
     public function getAuthConfigSource()
     {
         return $this->authConfigSource;
@@ -129,17 +172,27 @@ class Config
     /**
      * Merges new config values with the existing ones (overriding)
      *
-     * @param array $config
+     * @param array{config?: array<string, mixed>, repositories?: array<mixed>} $config
+     * @param string $source
+     *
+     * @return void
      */
-    public function merge($config)
+    public function merge($config, $source = self::SOURCE_UNKNOWN)
     {
         // override defaults with given config
         if (!empty($config['config']) && is_array($config['config'])) {
             foreach ($config['config'] as $key => $val) {
-                if (in_array($key, array('bitbucket-oauth', 'github-oauth', 'gitlab-oauth', 'gitlab-token', 'http-basic', 'bearer')) && isset($this->config[$key])) {
+                if (in_array($key, array('bitbucket-oauth', 'github-oauth', 'gitlab-oauth', 'gitlab-token', 'http-basic', 'bearer'), true) && isset($this->config[$key])) {
                     $this->config[$key] = array_merge($this->config[$key], $val);
-                } elseif (in_array($key, array('gitlab-domains', 'github-domains')) && isset($this->config[$key])) {
+                    $this->setSourceOfConfigValue($val, $key, $source);
+                } elseif (in_array($key, array('allow-plugins'), true) && isset($this->config[$key]) && is_array($this->config[$key])) {
+                    // merging $val first to get the local config on top of the global one, then appending the global config,
+                    // then merging local one again to make sure the values from local win over global ones for keys present in both
+                    $this->config[$key] = array_merge($val, $this->config[$key], $val);
+                    $this->setSourceOfConfigValue($val, $key, $source);
+                } elseif (in_array($key, array('gitlab-domains', 'github-domains'), true) && isset($this->config[$key])) {
                     $this->config[$key] = array_unique(array_merge($this->config[$key], $val));
+                    $this->setSourceOfConfigValue($val, $key, $source);
                 } elseif ('preferred-install' === $key && isset($this->config[$key])) {
                     if (is_array($val) || is_array($this->config[$key])) {
                         if (is_string($val)) {
@@ -147,8 +200,10 @@ class Config
                         }
                         if (is_string($this->config[$key])) {
                             $this->config[$key] = array('*' => $this->config[$key]);
+                            $this->sourceOfConfigValue[$key . '*'] = $source;
                         }
                         $this->config[$key] = array_merge($this->config[$key], $val);
+                        $this->setSourceOfConfigValue($val, $key, $source);
                         // the full match pattern needs to be last
                         if (isset($this->config[$key]['*'])) {
                             $wildcard = $this->config[$key]['*'];
@@ -157,9 +212,11 @@ class Config
                         }
                     } else {
                         $this->config[$key] = $val;
+                        $this->setSourceOfConfigValue($val, $key, $source);
                     }
                 } else {
                     $this->config[$key] = $val;
+                    $this->setSourceOfConfigValue($val, $key, $source);
                 }
             }
         }
@@ -170,24 +227,32 @@ class Config
             foreach ($newRepos as $name => $repository) {
                 // disable a repository by name
                 if (false === $repository) {
-                    $this->disableRepoByName($name);
+                    $this->disableRepoByName((string) $name);
                     continue;
                 }
 
                 // disable a repository with an anonymous {"name": false} repo
                 if (is_array($repository) && 1 === count($repository) && false === current($repository)) {
-                    $this->disableRepoByName(key($repository));
+                    $this->disableRepoByName((string) key($repository));
                     continue;
                 }
 
+                // auto-deactivate the default packagist.org repo if it gets redefined
+                if (isset($repository['type'], $repository['url']) && $repository['type'] === 'composer' && Preg::isMatch('{^https?://(?:[a-z0-9-.]+\.)?packagist.org(/|$)}', $repository['url'])) {
+                    $this->disableRepoByName('packagist.org');
+                }
+
                 // store repo
                 if (is_int($name)) {
                     $this->repositories[] = $repository;
+                    $this->setSourceOfConfigValue($repository, 'repositories.' . array_search($repository, $this->repositories, true), $source);
                 } else {
                     if ($name === 'packagist') { // BC support for default "packagist" named repo
                         $this->repositories[$name . '.org'] = $repository;
+                        $this->setSourceOfConfigValue($repository, 'repositories.' . $name . '.org', $source);
                     } else {
                         $this->repositories[$name] = $repository;
+                        $this->setSourceOfConfigValue($repository, 'repositories.' . $name, $source);
                     }
                 }
             }
@@ -196,7 +261,7 @@ class Config
     }
 
     /**
-     * @return array
+     * @return array<int|string, mixed>
      */
     public function getRepositories()
     {
@@ -209,6 +274,7 @@ class Config
      * @param  string            $key
      * @param  int               $flags Options (see class constants)
      * @throws \RuntimeException
+     *
      * @return mixed
      */
     public function get($key, $flags = 0)
@@ -229,6 +295,10 @@ class Config
                 $env = 'COMPOSER_' . strtoupper(strtr($key, '-', '_'));
 
                 $val = $this->getComposerEnv($env);
+                if ($val !== false) {
+                    $this->setSourceOfConfigValue($val, $key, $env);
+                }
+
                 $val = rtrim((string) $this->process(false !== $val ? $val : $this->config[$key], $flags), '/\\');
                 $val = Platform::expandPath($val);
 
@@ -247,6 +317,8 @@ class Config
                 $val = $this->getComposerEnv($env);
                 if (false === $val) {
                     $val = $this->config[$key];
+                } else {
+                    $this->setSourceOfConfigValue($val, $key, $env);
                 }
 
                 return $val !== 'false' && (bool) $val;
@@ -269,7 +341,7 @@ class Config
 
             // numbers with kb/mb/gb support, without env var support
             case 'cache-files-maxsize':
-                if (!preg_match('/^\s*([0-9.]+)\s*(?:([kmg])(?:i?b)?)?\s*$/i', $this->config[$key], $matches)) {
+                if (!Preg::isMatch('/^\s*([0-9.]+)\s*(?:([kmg])(?:i?b)?)?\s*$/i', $this->config[$key], $matches)) {
                     throw new \RuntimeException(
                         "Could not parse the value of '$key': {$this->config[$key]}"
                     );
@@ -302,19 +374,23 @@ class Config
                 return (int) $this->config['cache-ttl'];
 
             case 'home':
-                $val = preg_replace('#^(\$HOME|~)(/|$)#', rtrim(getenv('HOME') ?: getenv('USERPROFILE'), '/\\') . '/', $this->config[$key]);
+                $val = Preg::replace('#^(\$HOME|~)(/|$)#', rtrim(Platform::getEnv('HOME') ?: Platform::getEnv('USERPROFILE'), '/\\') . '/', $this->config[$key]);
 
                 return rtrim($this->process($val, $flags), '/\\');
 
             case 'bin-compat':
                 $value = $this->getComposerEnv('COMPOSER_BIN_COMPAT') ?: $this->config[$key];
 
-                if (!in_array($value, array('auto', 'full'))) {
+                if (!in_array($value, array('auto', 'full', 'proxy', 'symlink'))) {
                     throw new \RuntimeException(
-                        "Invalid value for 'bin-compat': {$value}. Expected auto, full"
+                        "Invalid value for 'bin-compat': {$value}. Expected auto, full or proxy"
                     );
                 }
 
+                if ($value === 'symlink') {
+                    trigger_error('config.bin-compat "symlink" is deprecated since Composer 2.2, use auto, full (for Windows compatibility) or proxy instead.', E_USER_DEPRECATED);
+                }
+
                 return $value;
 
             case 'discard-changes':
@@ -360,6 +436,11 @@ class Config
         }
     }
 
+    /**
+     * @param int $flags
+     *
+     * @return array<string, mixed[]>
+     */
     public function all($flags = 0)
     {
         $all = array(
@@ -372,6 +453,38 @@ class Config
         return $all;
     }
 
+    /**
+     * @param string $key
+     * @return string
+     */
+    public function getSourceOfValue($key)
+    {
+        $this->get($key);
+
+        return isset($this->sourceOfConfigValue[$key]) ? $this->sourceOfConfigValue[$key] : self::SOURCE_UNKNOWN;
+    }
+
+    /**
+     * @param mixed  $configValue
+     * @param string $path
+     * @param string $source
+     *
+     * @return void
+     */
+    private function setSourceOfConfigValue($configValue, $path, $source)
+    {
+        $this->sourceOfConfigValue[$path] = $source;
+
+        if (is_array($configValue)) {
+            foreach ($configValue as $key => $value) {
+                $this->setSourceOfConfigValue($value, $path . '.' . $key, $source);
+            }
+        }
+    }
+
+    /**
+     * @return array<string, mixed[]>
+     */
     public function raw()
     {
         return array(
@@ -396,6 +509,7 @@ class Config
      *
      * @param  string|int|null $value a config string that can contain {$refs-to-other-config}
      * @param  int             $flags Options (see class constants)
+     *
      * @return string|int|null
      */
     private function process($value, $flags)
@@ -406,7 +520,7 @@ class Config
             return $value;
         }
 
-        return preg_replace_callback('#\{\$(.+)\}#', function ($match) use ($config, $flags) {
+        return Preg::replaceCallback('#\{\$(.+)\}#', function ($match) use ($config, $flags) {
             return $config->get($match[1], $flags);
         }, $value);
     }
@@ -421,11 +535,11 @@ class Config
      */
     private function realpath($path)
     {
-        if (preg_match('{^(?:/|[a-z]:|[a-z0-9.]+://)}i', $path)) {
+        if (Preg::isMatch('{^(?:/|[a-z]:|[a-z0-9.]+://)}i', $path)) {
             return $path;
         }
 
-        return $this->baseDir . '/' . $path;
+        return $this->baseDir ? $this->baseDir . '/' . $path : $path;
     }
 
     /**
@@ -440,12 +554,17 @@ class Config
     private function getComposerEnv($var)
     {
         if ($this->useEnvironment) {
-            return getenv($var);
+            return Platform::getEnv($var);
         }
 
         return false;
     }
 
+    /**
+     * @param string $name
+     *
+     * @return void
+     */
     private function disableRepoByName($name)
     {
         if (isset($this->repositories[$name])) {
@@ -460,6 +579,8 @@ class Config
      *
      * @param string      $url
      * @param IOInterface $io
+     *
+     * @return void
      */
     public function prohibitUrlByConfig($url, IOInterface $io = null)
     {
@@ -470,16 +591,27 @@ class Config
 
         // Extract scheme and throw exception on known insecure protocols
         $scheme = parse_url($url, PHP_URL_SCHEME);
+        $hostname = parse_url($url, PHP_URL_HOST);
         if (in_array($scheme, array('http', 'git', 'ftp', 'svn'))) {
             if ($this->get('secure-http')) {
+                if ($scheme === 'svn') {
+                    if (in_array($hostname, $this->get('secure-svn-domains'), true)) {
+                        return;
+                    }
+
+                    throw new TransportException("Your configuration does not allow connections to $url. See https://getcomposer.org/doc/06-config.md#secure-svn-domains for details.");
+                }
+
                 throw new TransportException("Your configuration does not allow connections to $url. See https://getcomposer.org/doc/06-config.md#secure-http for details.");
             }
             if ($io) {
                 $host = parse_url($url, PHP_URL_HOST);
-                if (!isset($this->warnedHosts[$host])) {
-                    $io->writeError("<warning>Warning: Accessing $host over $scheme which is an insecure protocol.</warning>");
+                if (is_string($host)) {
+                    if (!isset($this->warnedHosts[$host])) {
+                        $io->writeError("<warning>Warning: Accessing $host over $scheme which is an insecure protocol.</warning>");
+                    }
+                    $this->warnedHosts[$host] = true;
                 }
-                $this->warnedHosts[$host] = true;
             }
         }
     }
@@ -493,6 +625,8 @@ class Config
      *     "vendor/bin/long-running-script --watch"
      *   ]
      * }
+     *
+     * @return void
      */
     public static function disableProcessTimeout()
     {
