@@ -12,6 +12,7 @@
 
 namespace Composer\DependencyResolver;
 
+use Composer\Repository\InstalledRepositoryInterface;
 use Composer\Repository\RepositoryInterface;
 
 /**
@@ -20,7 +21,7 @@ use Composer\Repository\RepositoryInterface;
  */
 class LocalRepoTransaction extends Transaction
 {
-    public function __construct(RepositoryInterface $lockedRepository, $localRepository)
+    public function __construct(RepositoryInterface $lockedRepository, InstalledRepositoryInterface $localRepository)
     {
         parent::__construct(
             $localRepository->getPackages(),
