@@ -12,9 +12,11 @@
 
 namespace Composer\DependencyResolver;
 
+use Composer\Filter\PlatformRequirementFilter\IgnoreListPlatformRequirementFilter;
+use Composer\Filter\PlatformRequirementFilter\PlatformRequirementFilterFactory;
+use Composer\Filter\PlatformRequirementFilter\PlatformRequirementFilterInterface;
 use Composer\IO\IOInterface;
 use Composer\Package\BasePackage;
-use Composer\Repository\PlatformRepository;
 
 /**
  * @author Nils Adermann <naderman@naderman.de>
@@ -41,7 +43,7 @@ class Solver
 
     /** @var int */
     protected $propagateIndex;
-    /** @var array[] */
+    /** @var mixed[] */
     protected $branches = array();
     /** @var Problem[] */
     protected $problems = array();
@@ -166,14 +168,15 @@ class Solver
     }
 
     /**
-     * @param bool|string[] $ignorePlatformReqs
      * @return void
      */
-    protected function checkForRootRequireProblems(Request $request, $ignorePlatformReqs)
+    protected function checkForRootRequireProblems(Request $request, PlatformRequirementFilterInterface $platformRequirementFilter)
     {
         foreach ($request->getRequires() as $packageName => $constraint) {
-            if ((true === $ignorePlatformReqs || (is_array($ignorePlatformReqs) && in_array($packageName, $ignorePlatformReqs, true))) && PlatformRepository::isPlatformPackage($packageName)) {
+            if ($platformRequirementFilter->isIgnored($packageName)) {
                 continue;
+            } elseif ($platformRequirementFilter instanceof IgnoreListPlatformRequirementFilter) {
+                $constraint = $platformRequirementFilter->filterConstraint($packageName, $constraint);
             }
 
             if (!$this->pool->whatProvides($packageName, $constraint)) {
@@ -185,18 +188,19 @@ class Solver
     }
 
     /**
-     * @param  bool|string[]      $ignorePlatformReqs
      * @return LockTransaction
      */
-    public function solve(Request $request, $ignorePlatformReqs = false)
+    public function solve(Request $request, PlatformRequirementFilterInterface $platformRequirementFilter = null)
     {
+        $platformRequirementFilter = $platformRequirementFilter ?: PlatformRequirementFilterFactory::ignoreNothing();
+
         $this->setupFixedMap($request);
 
         $this->io->writeError('Generating rules', true, IOInterface::DEBUG);
         $ruleSetGenerator = new RuleSetGenerator($this->policy, $this->pool);
-        $this->rules = $ruleSetGenerator->getRulesFor($request, $ignorePlatformReqs);
+        $this->rules = $ruleSetGenerator->getRulesFor($request, $platformRequirementFilter);
         unset($ruleSetGenerator);
-        $this->checkForRootRequireProblems($request, $ignorePlatformReqs);
+        $this->checkForRootRequireProblems($request, $platformRequirementFilter);
         $this->decisions = new Decisions($this->pool);
         $this->watchGraph = new RuleWatchGraph;
 
@@ -320,7 +324,7 @@ class Solver
 
             if ($newLevel <= 0 || $newLevel >= $level) {
                 throw new SolverBugException(
-                    "Trying to revert to invalid level ".(int) $newLevel." from level ".(int) $level."."
+                    "Trying to revert to invalid level ".$newLevel." from level ".$level."."
                 );
             }
 
@@ -419,7 +423,7 @@ class Solver
             while ($l1retry) {
                 $l1retry = false;
 
-                if (!$num && !--$l1num) {
+                if (0 === $num && 0 === --$l1num) {
                     // all level 1 literals done
                     break 2;
                 }
@@ -443,7 +447,7 @@ class Solver
 
                 unset($seen[abs($literal)]);
 
-                if ($num && 0 === --$num) {
+                if (0 !== $num && 0 === --$num) {
                     if ($literal < 0) {
                         $this->testFlagLearnedPositiveLiteral = true;
                     }
