@@ -17,6 +17,8 @@ use Composer\Repository\RepositorySet;
 
 /**
  * @author Nils Adermann <naderman@naderman.de>
+ *
+ * @method self::ERROR_DEPENDENCY_RESOLUTION_FAILED getCode()
  */
 class SolverProblemsException extends \RuntimeException
 {
@@ -47,16 +49,14 @@ class SolverProblemsException extends \RuntimeException
     public function getPrettyString(RepositorySet $repositorySet, Request $request, Pool $pool, $isVerbose, $isDevExtraction = false)
     {
         $installedMap = $request->getPresentMap(true);
-        $hasExtensionProblems = false;
+        $missingExtensions = array();
         $isCausedByLock = false;
 
         $problems = array();
         foreach ($this->problems as $problem) {
             $problems[] = $problem->getPrettyString($repositorySet, $request, $pool, $isVerbose, $installedMap, $this->learnedPool)."\n";
 
-            if (!$hasExtensionProblems && $this->hasExtensionProblems($problem->getReasons())) {
-                $hasExtensionProblems = true;
-            }
+            $missingExtensions = array_merge($missingExtensions, $this->getExtensionProblems($problem->getReasons()));
 
             $isCausedByLock = $isCausedByLock || $problem->isCausedByLock($repositorySet, $request, $pool);
         }
@@ -72,8 +72,8 @@ class SolverProblemsException extends \RuntimeException
             $hints[] = "Potential causes:\n - A typo in the package name\n - The package is not available in a stable-enough version according to your minimum-stability setting\n   see <https://getcomposer.org/doc/04-schema.md#minimum-stability> for more details.\n - It's a private package and you forgot to add a custom repository to find it\n\nRead <https://getcomposer.org/doc/articles/troubleshooting.md> for further common problems.";
         }
 
-        if ($hasExtensionProblems) {
-            $hints[] = $this->createExtensionHint();
+        if (!empty($missingExtensions)) {
+            $hints[] = $this->createExtensionHint($missingExtensions);
         }
 
         if ($isCausedByLock && !$isDevExtraction && !$request->getUpdateAllowTransitiveRootDependencies()) {
@@ -106,9 +106,10 @@ class SolverProblemsException extends \RuntimeException
     }
 
     /**
+     * @param string[] $missingExtensions
      * @return string
      */
-    private function createExtensionHint()
+    private function createExtensionHint(array $missingExtensions)
     {
         $paths = IniHelper::getAll();
 
@@ -116,28 +117,34 @@ class SolverProblemsException extends \RuntimeException
             return '';
         }
 
+        $ignoreExtensionsArguments = implode(" ", array_map(function ($extension) {
+            return "--ignore-platform-req=$extension";
+        }, $missingExtensions));
+
         $text = "To enable extensions, verify that they are enabled in your .ini files:\n    - ";
         $text .= implode("\n    - ", $paths);
-        $text .= "\nYou can also run `php --ini` inside terminal to see which files are used by PHP in CLI mode.";
+        $text .= "\nYou can also run `php --ini` in a terminal to see which files are used by PHP in CLI mode.";
+        $text .= "\nAlternatively, you can run Composer with `$ignoreExtensionsArguments` to temporarily ignore these required extensions.";
 
         return $text;
     }
 
     /**
      * @param Rule[][] $reasonSets
-     * @return bool
+     * @return string[]
      */
-    private function hasExtensionProblems(array $reasonSets)
+    private function getExtensionProblems(array $reasonSets)
     {
+        $missingExtensions = array();
         foreach ($reasonSets as $reasonSet) {
             foreach ($reasonSet as $rule) {
                 $required = $rule->getRequiredPackage();
                 if (null !== $required && 0 === strpos($required, 'ext-')) {
-                    return true;
+                    $missingExtensions[$required] = 1;
                 }
             }
         }
 
-        return false;
+        return array_keys($missingExtensions);
     }
 }
