@@ -22,7 +22,7 @@ use Composer\Package\PackageInterface;
 class PharDownloader extends ArchiveDownloader
 {
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     protected function extract(PackageInterface $package, $file, $path)
     {
@@ -34,5 +34,7 @@ class PharDownloader extends ArchiveDownloader
          * https://github.com/koto/phar-util
          * http://blog.kotowicz.net/2010/08/hardening-php-how-to-securely-include.html
          */
+
+        return \React\Promise\resolve();
     }
 }
