@@ -16,6 +16,8 @@ use Composer\DependencyResolver\Transaction;
 use Composer\Installer\InstallerEvent;
 use Composer\IO\IOInterface;
 use Composer\Composer;
+use Composer\Pcre\Preg;
+use Composer\Util\Platform;
 use Composer\DependencyResolver\Operation\OperationInterface;
 use Composer\Repository\RepositoryInterface;
 use Composer\Script;
@@ -23,8 +25,9 @@ use Composer\Installer\PackageEvent;
 use Composer\Installer\BinaryInstaller;
 use Composer\Util\ProcessExecutor;
 use Composer\Script\Event as ScriptEvent;
-use Composer\ClassLoader;
+use Composer\Autoload\ClassLoader;
 use Symfony\Component\Process\PhpExecutableFinder;
+use Symfony\Component\Process\ExecutableFinder;
 
 /**
  * The Event Dispatcher.
@@ -51,6 +54,8 @@ class EventDispatcher
     protected $process;
     /** @var array<string, array<int, array<callable|string>>> */
     protected $listeners = array();
+    /** @var bool */
+    protected $runScripts = true;
     /** @var list<string> */
     private $eventStack;
 
@@ -69,6 +74,19 @@ class EventDispatcher
         $this->eventStack = array();
     }
 
+    /**
+     * Set whether script handlers are active or not
+     *
+     * @param bool $runScripts
+     * @return $this
+     */
+    public function setRunScripts($runScripts = true)
+    {
+        $this->runScripts = (bool) $runScripts;
+
+        return $this;
+    }
+
     /**
      * Dispatch an event
      *
@@ -89,12 +107,12 @@ class EventDispatcher
     /**
      * Dispatch a script event.
      *
-     * @param  string $eventName      The constant in ScriptEvents
-     * @param  bool   $devMode
-     * @param  array  $additionalArgs Arguments passed by the user
-     * @param  array  $flags          Optional flags to pass data not as argument
-     * @return int    return code of the executed script if any, for php scripts a false return
-     *                               value is changed to 1, anything else to 0
+     * @param  string               $eventName      The constant in ScriptEvents
+     * @param  bool                 $devMode
+     * @param  array<int, mixed>    $additionalArgs Arguments passed by the user
+     * @param  array<string, mixed> $flags          Optional flags to pass data not as argument
+     * @return int                                  return code of the executed script if any, for php scripts a false return
+     *                                              value is changed to 1, anything else to 0
      */
     public function dispatchScript($eventName, $devMode = false, $additionalArgs = array(), $flags = array())
     {
@@ -104,11 +122,11 @@ class EventDispatcher
     /**
      * Dispatch a package event.
      *
-     * @param string              $eventName  The constant in PackageEvents
-     * @param bool                $devMode    Whether or not we are in dev mode
-     * @param RepositoryInterface $localRepo  The installed repository
-     * @param array               $operations The list of operations
-     * @param OperationInterface  $operation  The package being installed/updated/removed
+     * @param string               $eventName  The constant in PackageEvents
+     * @param bool                 $devMode    Whether or not we are in dev mode
+     * @param RepositoryInterface  $localRepo  The installed repository
+     * @param OperationInterface[] $operations The list of operations
+     * @param OperationInterface   $operation  The package being installed/updated/removed
      *
      * @return int return code of the executed script if any, for php scripts a false return
      *             value is changed to 1, anything else to 0
@@ -144,7 +162,7 @@ class EventDispatcher
      */
     protected function doDispatch(Event $event)
     {
-        if (getenv('COMPOSER_DEBUG_EVENTS')) {
+        if (Platform::getEnv('COMPOSER_DEBUG_EVENTS')) {
             $details = null;
             if ($event instanceof PackageEvent) {
                 $details = (string) $event->getOperation();
@@ -156,133 +174,175 @@ class EventDispatcher
 
         $this->pushEvent($event);
 
-        $return = 0;
-        foreach ($listeners as $callable) {
-            $this->ensureBinDirIsInPath();
+        try {
+            $returnMax = 0;
+            foreach ($listeners as $callable) {
+                $return = 0;
+                $this->ensureBinDirIsInPath();
 
-            if (!is_string($callable)) {
-                if (!is_callable($callable)) {
-                    $className = is_object($callable[0]) ? get_class($callable[0]) : $callable[0];
+                if (!is_string($callable)) {
+                    if (!is_callable($callable)) {
+                        $className = is_object($callable[0]) ? get_class($callable[0]) : $callable[0];
 
-                    throw new \RuntimeException('Subscriber '.$className.'::'.$callable[1].' for event '.$event->getName().' is not callable, make sure the function is defined and public');
-                }
-                if (is_array($callable) && (is_string($callable[0]) || is_object($callable[0])) && is_string($callable[1])) {
-                    $this->io->writeError(sprintf('> %s: %s', $event->getName(), (is_object($callable[0]) ? get_class($callable[0]) : $callable[0]).'->'.$callable[1]), true, IOInterface::VERBOSE);
-                }
-                $return = false === call_user_func($callable, $event) ? 1 : 0;
-            } elseif ($this->isComposerScript($callable)) {
-                $this->io->writeError(sprintf('> %s: %s', $event->getName(), $callable), true, IOInterface::VERBOSE);
+                        throw new \RuntimeException('Subscriber '.$className.'::'.$callable[1].' for event '.$event->getName().' is not callable, make sure the function is defined and public');
+                    }
+                    if (is_array($callable) && (is_string($callable[0]) || is_object($callable[0])) && is_string($callable[1])) {
+                        $this->io->writeError(sprintf('> %s: %s', $event->getName(), (is_object($callable[0]) ? get_class($callable[0]) : $callable[0]).'->'.$callable[1]), true, IOInterface::VERBOSE);
+                    }
+                    $return = false === call_user_func($callable, $event) ? 1 : 0;
+                } elseif ($this->isComposerScript($callable)) {
+                    $this->io->writeError(sprintf('> %s: %s', $event->getName(), $callable), true, IOInterface::VERBOSE);
 
-                $script = explode(' ', substr($callable, 1));
-                $scriptName = $script[0];
-                unset($script[0]);
+                    $script = explode(' ', substr($callable, 1));
+                    $scriptName = $script[0];
+                    unset($script[0]);
+
+                    $args = array_merge($script, $event->getArguments());
+                    $flags = $event->getFlags();
+                    if (strpos($callable, '@composer ') === 0) {
+                        $exec = $this->getPhpExecCommand() . ' ' . ProcessExecutor::escape(Platform::getEnv('COMPOSER_BINARY')) . ' ' . implode(' ', $args);
+                        if (0 !== ($exitCode = $this->executeTty($exec))) {
+                            $this->io->writeError(sprintf('<error>Script %s handling the %s event returned with error code '.$exitCode.'</error>', $callable, $event->getName()), true, IOInterface::QUIET);
+
+                            throw new ScriptExecutionException('Error Output: '.$this->process->getErrorOutput(), $exitCode);
+                        }
+                    } else {
+                        if (!$this->getListeners(new Event($scriptName))) {
+                            $this->io->writeError(sprintf('<warning>You made a reference to a non-existent script %s</warning>', $callable), true, IOInterface::QUIET);
+                        }
+
+                        try {
+                            /** @var InstallerEvent $event */
+                            $scriptEvent = new Script\Event($scriptName, $event->getComposer(), $event->getIO(), $event->isDevMode(), $args, $flags);
+                            $scriptEvent->setOriginatingEvent($event);
+                            $return = $this->dispatch($scriptName, $scriptEvent);
+                        } catch (ScriptExecutionException $e) {
+                            $this->io->writeError(sprintf('<error>Script %s was called via %s</error>', $callable, $event->getName()), true, IOInterface::QUIET);
+                            throw $e;
+                        }
+                    }
+                } elseif ($this->isPhpScript($callable)) {
+                    $className = substr($callable, 0, strpos($callable, '::'));
+                    $methodName = substr($callable, strpos($callable, '::') + 2);
+
+                    if (!class_exists($className)) {
+                        $this->io->writeError('<warning>Class '.$className.' is not autoloadable, can not call '.$event->getName().' script</warning>', true, IOInterface::QUIET);
+                        continue;
+                    }
+                    if (!is_callable($callable)) {
+                        $this->io->writeError('<warning>Method '.$callable.' is not callable, can not call '.$event->getName().' script</warning>', true, IOInterface::QUIET);
+                        continue;
+                    }
+
+                    try {
+                        $return = false === $this->executeEventPhpScript($className, $methodName, $event) ? 1 : 0;
+                    } catch (\Exception $e) {
+                        $message = "Script %s handling the %s event terminated with an exception";
+                        $this->io->writeError('<error>'.sprintf($message, $callable, $event->getName()).'</error>', true, IOInterface::QUIET);
+                        throw $e;
+                    }
+                } else {
+                    $args = implode(' ', array_map(array('Composer\Util\ProcessExecutor', 'escape'), $event->getArguments()));
+                    $exec = $callable . ($args === '' ? '' : ' '.$args);
+                    if ($this->io->isVerbose()) {
+                        $this->io->writeError(sprintf('> %s: %s', $event->getName(), $exec));
+                    } elseif ($event->getName() !== '__exec_command') {
+                        // do not output the command being run when using `composer exec` as it is fairly obvious the user is running it
+                        $this->io->writeError(sprintf('> %s', $exec));
+                    }
+
+                    $possibleLocalBinaries = $this->composer->getPackage()->getBinaries();
+                    if ($possibleLocalBinaries) {
+                        foreach ($possibleLocalBinaries as $localExec) {
+                            if (Preg::isMatch('{\b'.preg_quote($callable).'$}', $localExec)) {
+                                $caller = BinaryInstaller::determineBinaryCaller($localExec);
+                                $exec = Preg::replace('{^'.preg_quote($callable).'}', $caller . ' ' . $localExec, $exec);
+                                break;
+                            }
+                        }
+                    }
+
+                    if (strpos($exec, '@putenv ') === 0) {
+                        if (false === strpos($exec, '=')) {
+                            Platform::clearEnv(substr($exec, 8));
+                        } else {
+                            list($var, $value) = explode('=', substr($exec, 8), 2);
+                            Platform::putEnv($var, $value);
+                        }
+
+                        continue;
+                    }
+                    if (strpos($exec, '@php ') === 0) {
+                        $pathAndArgs = substr($exec, 5);
+                        if (Platform::isWindows()) {
+                            $pathAndArgs = Preg::replaceCallback('{^\S+}', function ($path) {
+                                return str_replace('/', '\\', $path[0]);
+                            }, $pathAndArgs);
+                        }
+                        // match somename (not in quote, and not a qualified path) and if it is not a valid path from CWD then try to find it
+                        // in $PATH. This allows support for `@php foo` where foo is a binary name found in PATH but not an actual relative path
+                        $matched = Preg::isMatch('{^[^\'"\s/\\\\]+}', $pathAndArgs, $match);
+                        if ($matched && !file_exists($match[0])) {
+                            $finder = new ExecutableFinder;
+                            if ($pathToExec = $finder->find($match[0])) {
+                                $pathAndArgs = $pathToExec . substr($pathAndArgs, strlen($match[0]));
+                            }
+                        }
+                        $exec = $this->getPhpExecCommand() . ' ' . $pathAndArgs;
+                    } else {
+                        $finder = new PhpExecutableFinder();
+                        $phpPath = $finder->find(false);
+                        if ($phpPath) {
+                            Platform::putEnv('PHP_BINARY', $phpPath);
+                        }
+
+                        if (Platform::isWindows()) {
+                            $exec = Preg::replaceCallback('{^\S+}', function ($path) {
+                                return str_replace('/', '\\', $path[0]);
+                            }, $exec);
+                        }
+                    }
+
+                    // if composer is being executed, make sure it runs the expected composer from current path
+                    // resolution, even if bin-dir contains composer too because the project requires composer/composer
+                    // see https://github.com/composer/composer/issues/8748
+                    if (strpos($exec, 'composer ') === 0) {
+                        $exec = $this->getPhpExecCommand() . ' ' . ProcessExecutor::escape(Platform::getEnv('COMPOSER_BINARY')) . substr($exec, 8);
+                    }
 
-                $args = array_merge($script, $event->getArguments());
-                $flags = $event->getFlags();
-                if (strpos($callable, '@composer ') === 0) {
-                    $exec = $this->getPhpExecCommand() . ' ' . ProcessExecutor::escape(getenv('COMPOSER_BINARY')) . ' ' . implode(' ', $args);
                     if (0 !== ($exitCode = $this->executeTty($exec))) {
                         $this->io->writeError(sprintf('<error>Script %s handling the %s event returned with error code '.$exitCode.'</error>', $callable, $event->getName()), true, IOInterface::QUIET);
 
                         throw new ScriptExecutionException('Error Output: '.$this->process->getErrorOutput(), $exitCode);
                     }
-                } else {
-                    if (!$this->getListeners(new Event($scriptName))) {
-                        $this->io->writeError(sprintf('<warning>You made a reference to a non-existent script %s</warning>', $callable), true, IOInterface::QUIET);
-                    }
-
-                    try {
-                        /** @var InstallerEvent $event */
-                        $scriptEvent = new Script\Event($scriptName, $event->getComposer(), $event->getIO(), $event->isDevMode(), $args, $flags);
-                        $scriptEvent->setOriginatingEvent($event);
-                        $return = $this->dispatch($scriptName, $scriptEvent);
-                    } catch (ScriptExecutionException $e) {
-                        $this->io->writeError(sprintf('<error>Script %s was called via %s</error>', $callable, $event->getName()), true, IOInterface::QUIET);
-                        throw $e;
-                    }
-                }
-            } elseif ($this->isPhpScript($callable)) {
-                $className = substr($callable, 0, strpos($callable, '::'));
-                $methodName = substr($callable, strpos($callable, '::') + 2);
-
-                if (!class_exists($className)) {
-                    $this->io->writeError('<warning>Class '.$className.' is not autoloadable, can not call '.$event->getName().' script</warning>', true, IOInterface::QUIET);
-                    continue;
-                }
-                if (!is_callable($callable)) {
-                    $this->io->writeError('<warning>Method '.$callable.' is not callable, can not call '.$event->getName().' script</warning>', true, IOInterface::QUIET);
-                    continue;
                 }
 
-                try {
-                    $return = false === $this->executeEventPhpScript($className, $methodName, $event) ? 1 : 0;
-                } catch (\Exception $e) {
-                    $message = "Script %s handling the %s event terminated with an exception";
-                    $this->io->writeError('<error>'.sprintf($message, $callable, $event->getName()).'</error>', true, IOInterface::QUIET);
-                    throw $e;
-                }
-            } else {
-                $args = implode(' ', array_map(array('Composer\Util\ProcessExecutor', 'escape'), $event->getArguments()));
-                $exec = $callable . ($args === '' ? '' : ' '.$args);
-                if ($this->io->isVerbose()) {
-                    $this->io->writeError(sprintf('> %s: %s', $event->getName(), $exec));
-                } else {
-                    $this->io->writeError(sprintf('> %s', $exec));
-                }
+                $returnMax = max($returnMax, $return);
 
-                $possibleLocalBinaries = $this->composer->getPackage()->getBinaries();
-                if ($possibleLocalBinaries) {
-                    foreach ($possibleLocalBinaries as $localExec) {
-                        if (preg_match('{\b'.preg_quote($callable).'$}', $localExec)) {
-                            $caller = BinaryInstaller::determineBinaryCaller($localExec);
-                            $exec = preg_replace('{^'.preg_quote($callable).'}', $caller . ' ' . $localExec, $exec);
-                            break;
-                        }
-                    }
-                }
-
-                if (strpos($exec, '@putenv ') === 0) {
-                    putenv(substr($exec, 8));
-                    list($var, $value) = explode('=', substr($exec, 8), 2);
-                    $_SERVER[$var] = $value;
-
-                    continue;
-                }
-                if (strpos($exec, '@php ') === 0) {
-                    $exec = $this->getPhpExecCommand() . ' ' . substr($exec, 5);
-                } else {
-                    $finder = new PhpExecutableFinder();
-                    $phpPath = $finder->find(false);
-                    if ($phpPath) {
-                        $_SERVER['PHP_BINARY'] = $phpPath;
-                        putenv('PHP_BINARY=' . $_SERVER['PHP_BINARY']);
-                    }
-                }
-
-                // if composer is being executed, make sure it runs the expected composer from current path
-                // resolution, even if bin-dir contains composer too because the project requires composer/composer
-                // see https://github.com/composer/composer/issues/8748
-                if (strpos($exec, 'composer ') === 0) {
-                    $exec = $this->getPhpExecCommand() . ' ' . ProcessExecutor::escape(getenv('COMPOSER_BINARY')) . substr($exec, 8);
-                }
-
-                if (0 !== ($exitCode = $this->executeTty($exec))) {
-                    $this->io->writeError(sprintf('<error>Script %s handling the %s event returned with error code '.$exitCode.'</error>', $callable, $event->getName()), true, IOInterface::QUIET);
-
-                    throw new ScriptExecutionException('Error Output: '.$this->process->getErrorOutput(), $exitCode);
+                if ($event->isPropagationStopped()) {
+                    break;
                 }
             }
+        } catch (\Exception $e) { // TODO Composer 2.2 turn all this into a finally
+            $this->popEvent();
 
-            if ($event->isPropagationStopped()) {
-                break;
-            }
+            throw $e;
+        } catch (\Throwable $e) {
+            $this->popEvent();
+
+            throw $e;
         }
 
         $this->popEvent();
 
-        return $return;
+        return $returnMax;
     }
 
+    /**
+     * @param string $exec
+     *
+     * @return int
+     */
     protected function executeTty($exec)
     {
         if ($this->io->isInteractive()) {
@@ -292,6 +352,9 @@ class EventDispatcher
         return $this->process->execute($exec);
     }
 
+    /**
+     * @return string
+     */
     protected function getPhpExecCommand()
     {
         $finder = new PhpExecutableFinder();
@@ -312,6 +375,8 @@ class EventDispatcher
      * @param string $className
      * @param string $methodName
      * @param Event  $event      Event invoking the PHP callable
+     *
+     * @return mixed
      */
     protected function executeEventPhpScript($className, $methodName, Event $event)
     {
@@ -330,6 +395,8 @@ class EventDispatcher
      * @param string   $eventName The event name - typically a constant
      * @param callable $listener  A callable expecting an event argument
      * @param int      $priority  A higher value represents a higher priority
+     *
+     * @return void
      */
     public function addListener($eventName, $listener, $priority = 0)
     {
@@ -338,6 +405,8 @@ class EventDispatcher
 
     /**
      * @param callable|object $listener A callable or an object instance for which all listeners should be removed
+     *
+     * @return void
      */
     public function removeListener($listener)
     {
@@ -358,6 +427,8 @@ class EventDispatcher
      * @see EventSubscriberInterface
      *
      * @param EventSubscriberInterface $subscriber
+     *
+     * @return void
      */
     public function addSubscriber(EventSubscriberInterface $subscriber)
     {
@@ -378,11 +449,11 @@ class EventDispatcher
      * Retrieves all listeners for a given event
      *
      * @param  Event $event
-     * @return array All listeners: callables and scripts
+     * @return array<callable|string> All listeners: callables and scripts
      */
     protected function getListeners(Event $event)
     {
-        $scriptListeners = $this->getScriptListeners($event);
+        $scriptListeners = $this->runScripts ? $this->getScriptListeners($event) : array();
 
         if (!isset($this->listeners[$event->getName()][0])) {
             $this->listeners[$event->getName()][0] = array();
@@ -412,7 +483,7 @@ class EventDispatcher
      * Finds all listeners defined as scripts in the package
      *
      * @param  Event $event Event object
-     * @return array Listeners
+     * @return string[] Listeners
      */
     protected function getScriptListeners(Event $event)
     {
@@ -435,8 +506,8 @@ class EventDispatcher
         $packages = $this->composer->getRepositoryManager()->getLocalRepository()->getCanonicalPackages();
         $packageMap = $generator->buildPackageMap($this->composer->getInstallationManager(), $package, $packages);
         $map = $generator->parseAutoloads($packageMap, $package);
-        $this->loader = $generator->createLoader($map);
-        $this->loader->register();
+        $this->loader = $generator->createLoader($map, $this->composer->getConfig()->get('vendor-dir'));
+        $this->loader->register(false);
 
         return $scripts[$event->getName()];
     }
@@ -483,27 +554,30 @@ class EventDispatcher
     /**
      * Pops the active event from the stack
      *
-     * @return mixed
+     * @return string|null
      */
     protected function popEvent()
     {
         return array_pop($this->eventStack);
     }
 
+    /**
+     * @return void
+     */
     private function ensureBinDirIsInPath()
     {
-        $pathStr = 'PATH';
-        if (!isset($_SERVER[$pathStr]) && isset($_SERVER['Path'])) {
-            $pathStr = 'Path';
+        $pathEnv = 'PATH';
+        if (false === Platform::getEnv('PATH') && false !== Platform::getEnv('Path')) {
+            $pathEnv = 'Path';
         }
 
         // add the bin dir to the PATH to make local binaries of deps usable in scripts
         $binDir = $this->composer->getConfig()->get('bin-dir');
         if (is_dir($binDir)) {
             $binDir = realpath($binDir);
-            if (isset($_SERVER[$pathStr]) && !preg_match('{(^|'.PATH_SEPARATOR.')'.preg_quote($binDir).'($|'.PATH_SEPARATOR.')}', $_SERVER[$pathStr])) {
-                $_SERVER[$pathStr] = $binDir.PATH_SEPARATOR.getenv($pathStr);
-                putenv($pathStr.'='.$_SERVER[$pathStr]);
+            $pathValue = Platform::getEnv($pathEnv);
+            if (!Preg::isMatch('{(^|'.PATH_SEPARATOR.')'.preg_quote($binDir).'($|'.PATH_SEPARATOR.')}', $pathValue)) {
+                Platform::putEnv($pathEnv, $binDir.PATH_SEPARATOR.$pathValue);
             }
         }
     }
