@@ -13,15 +13,16 @@
 namespace Composer\Package\Loader;
 
 use Composer\Package\BasePackage;
-use Composer\Package\AliasPackage;
 use Composer\Config;
 use Composer\IO\IOInterface;
-use Composer\Package\RootPackageInterface;
+use Composer\Package\RootAliasPackage;
+use Composer\Pcre\Preg;
 use Composer\Repository\RepositoryFactory;
 use Composer\Package\Version\VersionGuesser;
 use Composer\Package\Version\VersionParser;
 use Composer\Package\RootPackage;
 use Composer\Repository\RepositoryManager;
+use Composer\Util\Platform;
 use Composer\Util\ProcessExecutor;
 
 /**
@@ -58,13 +59,22 @@ class RootPackageLoader extends ArrayLoader
     }
 
     /**
-     * @param  array                $config package data
-     * @param  string               $class  FQCN to be instantiated
-     * @param  string               $cwd    cwd of the root package to be used to guess the version if it is not provided
-     * @return RootPackageInterface
+     * @inheritDoc
+     *
+     * @template PackageClass of RootPackage
+     *
+     * @param string|null $cwd
+     *
+     * @return RootPackage|RootAliasPackage
+     *
+     * @phpstan-param class-string<PackageClass> $class
      */
     public function load(array $config, $class = 'Composer\Package\RootPackage', $cwd = null)
     {
+        if ($class !== 'Composer\Package\RootPackage') {
+            trigger_error('The $class arg is deprecated, please reach out to Composer maintainers ASAP if you still need this.', E_USER_DEPRECATED);
+        }
+
         if (!isset($config['name'])) {
             $config['name'] = '__root__';
         } elseif ($err = ValidatingArrayLoader::hasPackageNamingError($config['name'])) {
@@ -75,8 +85,8 @@ class RootPackageLoader extends ArrayLoader
             $commit = null;
 
             // override with env var if available
-            if (getenv('COMPOSER_ROOT_VERSION')) {
-                $config['version'] = getenv('COMPOSER_ROOT_VERSION');
+            if (Platform::getEnv('COMPOSER_ROOT_VERSION')) {
+                $config['version'] = Platform::getEnv('COMPOSER_ROOT_VERSION');
             } else {
                 $versionData = $this->versionGuesser->guessVersion($config, $cwd ?: getcwd());
                 if ($versionData) {
@@ -105,9 +115,16 @@ class RootPackageLoader extends ArrayLoader
             }
         }
 
-        $realPackage = $package = parent::load($config, $class);
-        if ($realPackage instanceof AliasPackage) {
+        /** @var RootPackage|RootAliasPackage $package */
+        $package = parent::load($config, $class);
+        if ($package instanceof RootAliasPackage) {
             $realPackage = $package->getAliasOf();
+        } else {
+            $realPackage = $package;
+        }
+
+        if (!$realPackage instanceof RootPackage) {
+            throw new \LogicException('Expecting a Composer\Package\RootPackage at this point');
         }
 
         if ($autoVersioned) {
@@ -130,8 +147,8 @@ class RootPackageLoader extends ArrayLoader
                     $links[$link->getTarget()] = $link->getConstraint()->getPrettyString();
                 }
                 $aliases = $this->extractAliases($links, $aliases);
-                $stabilityFlags = $this->extractStabilityFlags($links, $stabilityFlags, $realPackage->getMinimumStability());
-                $references = $this->extractReferences($links, $references);
+                $stabilityFlags = self::extractStabilityFlags($links, $realPackage->getMinimumStability(), $stabilityFlags);
+                $references = self::extractReferences($links, $references);
 
                 if (isset($links[$config['name']])) {
                     throw new \RuntimeException(sprintf('Root package \'%s\' cannot require itself in its composer.json' . PHP_EOL .
@@ -171,10 +188,16 @@ class RootPackageLoader extends ArrayLoader
         return $package;
     }
 
+    /**
+     * @param array<string, string>                                                                  $requires
+     * @param list<array{package: string, version: string, alias: string, alias_normalized: string}> $aliases
+     *
+     * @return list<array{package: string, version: string, alias: string, alias_normalized: string}>
+     */
     private function extractAliases(array $requires, array $aliases)
     {
         foreach ($requires as $reqName => $reqVersion) {
-            if (preg_match('{^([^,\s#]+)(?:#[^ ]+)? +as +([^,\s]+)$}', $reqVersion, $match)) {
+            if (Preg::isMatch('{^([^,\s#]+)(?:#[^ ]+)? +as +([^,\s]+)$}', $reqVersion, $match)) {
                 $aliases[] = array(
                     'package' => strtolower($reqName),
                     'version' => $this->versionParser->normalize($match[1], $reqVersion),
@@ -189,17 +212,30 @@ class RootPackageLoader extends ArrayLoader
         return $aliases;
     }
 
-    private function extractStabilityFlags(array $requires, array $stabilityFlags, $minimumStability)
+    /**
+     * @internal
+     *
+     * @param array<string, string> $requires
+     * @param string                $minimumStability
+     * @param array<string, int>    $stabilityFlags
+     *
+     * @return array<string, int>
+     *
+     * @phpstan-param array<string, BasePackage::STABILITY_*> $stabilityFlags
+     * @phpstan-return array<string, BasePackage::STABILITY_*>
+     */
+    public static function extractStabilityFlags(array $requires, $minimumStability, array $stabilityFlags)
     {
         $stabilities = BasePackage::$stabilities;
+        /** @var int $minimumStability */
         $minimumStability = $stabilities[$minimumStability];
         foreach ($requires as $reqName => $reqVersion) {
             $constraints = array();
 
             // extract all sub-constraints in case it is an OR/AND multi-constraint
-            $orSplit = preg_split('{\s*\|\|?\s*}', trim($reqVersion));
+            $orSplit = Preg::split('{\s*\|\|?\s*}', trim($reqVersion));
             foreach ($orSplit as $orConstraint) {
-                $andSplit = preg_split('{(?<!^|as|[=>< ,]) *(?<!-)[, ](?!-) *(?!,|as|$)}', $orConstraint);
+                $andSplit = Preg::split('{(?<!^|as|[=>< ,]) *(?<!-)[, ](?!-) *(?!,|as|$)}', $orConstraint);
                 foreach ($andSplit as $andConstraint) {
                     $constraints[] = $andConstraint;
                 }
@@ -208,7 +244,7 @@ class RootPackageLoader extends ArrayLoader
             // parse explicit stability flags to the most unstable
             $match = false;
             foreach ($constraints as $constraint) {
-                if (preg_match('{^[^@]*?@('.implode('|', array_keys($stabilities)).')$}i', $constraint, $match)) {
+                if (Preg::isMatch('{^[^@]*?@('.implode('|', array_keys($stabilities)).')$}i', $constraint, $match)) {
                     $name = strtolower($reqName);
                     $stability = $stabilities[VersionParser::normalizeStability($match[1])];
 
@@ -227,8 +263,8 @@ class RootPackageLoader extends ArrayLoader
             foreach ($constraints as $constraint) {
                 // infer flags for requirements that have an explicit -dev or -beta version specified but only
                 // for those that are more unstable than the minimumStability or existing flags
-                $reqVersion = preg_replace('{^([^,\s@]+) as .+$}', '$1', $constraint);
-                if (preg_match('{^[^,\s@]+$}', $reqVersion) && 'stable' !== ($stabilityName = VersionParser::parseStability($reqVersion))) {
+                $reqVersion = Preg::replace('{^([^,\s@]+) as .+$}', '$1', $constraint);
+                if (Preg::isMatch('{^[^,\s@]+$}', $reqVersion) && 'stable' !== ($stabilityName = VersionParser::parseStability($reqVersion))) {
                     $name = strtolower($reqName);
                     $stability = $stabilities[$stabilityName];
                     if ((isset($stabilityFlags[$name]) && $stabilityFlags[$name] > $stability) || ($minimumStability > $stability)) {
@@ -242,11 +278,19 @@ class RootPackageLoader extends ArrayLoader
         return $stabilityFlags;
     }
 
-    private function extractReferences(array $requires, array $references)
+    /**
+     * @internal
+     *
+     * @param array<string, string> $requires
+     * @param array<string, string> $references
+     *
+     * @return array<string, string>
+     */
+    public static function extractReferences(array $requires, array $references)
     {
         foreach ($requires as $reqName => $reqVersion) {
-            $reqVersion = preg_replace('{^([^,\s@]+) as .+$}', '$1', $reqVersion);
-            if (preg_match('{^[^,\s@]+?#([a-f0-9]+)$}', $reqVersion, $match) && 'dev' === VersionParser::parseStability($reqVersion)) {
+            $reqVersion = Preg::replace('{^([^,\s@]+) as .+$}', '$1', $reqVersion);
+            if (Preg::isMatch('{^[^,\s@]+?#([a-f0-9]+)$}', $reqVersion, $match) && 'dev' === VersionParser::parseStability($reqVersion)) {
                 $name = strtolower($reqName);
                 $references[$name] = $match[1];
             }
