@@ -14,10 +14,13 @@ namespace Composer\Plugin;
 
 use Composer\Composer;
 use Composer\EventDispatcher\EventSubscriberInterface;
+use Composer\Installer\InstallerInterface;
 use Composer\IO\IOInterface;
+use Composer\Package\BasePackage;
 use Composer\Package\CompletePackage;
 use Composer\Package\Package;
 use Composer\Package\Version\VersionParser;
+use Composer\Pcre\Preg;
 use Composer\Repository\RepositoryInterface;
 use Composer\Repository\InstalledRepository;
 use Composer\Repository\RootPackageRepository;
@@ -48,9 +51,19 @@ class PluginManager
 
     /** @var array<PluginInterface> */
     protected $plugins = array();
-    /** @var array<string, PluginInterface> */
+    /** @var array<string, PluginInterface|InstallerInterface> */
     protected $registeredPlugins = array();
 
+    /**
+     * @var array<non-empty-string, bool>|null
+     */
+    private $allowPluginRules;
+
+    /**
+     * @var array<non-empty-string, bool>|null
+     */
+    private $allowGlobalPluginRules;
+
     /** @var int */
     private static $classCounter = 0;
 
@@ -69,6 +82,9 @@ class PluginManager
         $this->globalComposer = $globalComposer;
         $this->versionParser = new VersionParser();
         $this->disablePlugins = $disablePlugins;
+
+        $this->allowPluginRules = $this->parseAllowedPlugins($composer->getConfig()->get('allow-plugins'));
+        $this->allowGlobalPluginRules = $this->parseAllowedPlugins($globalComposer !== null ? $globalComposer->getConfig()->get('allow-plugins') : false);
     }
 
     /**
@@ -83,13 +99,32 @@ class PluginManager
         }
 
         $repo = $this->composer->getRepositoryManager()->getLocalRepository();
-        $globalRepo = $this->globalComposer ? $this->globalComposer->getRepositoryManager()->getLocalRepository() : null;
+        $globalRepo = $this->globalComposer !== null ? $this->globalComposer->getRepositoryManager()->getLocalRepository() : null;
         $this->loadRepository($repo, false);
         if ($globalRepo) {
             $this->loadRepository($globalRepo, true);
         }
     }
 
+    /**
+     * Deactivate all plugins from currently installed plugin packages
+     *
+     * @return void
+     */
+    public function deactivateInstalledPlugins()
+    {
+        if ($this->disablePlugins) {
+            return;
+        }
+
+        $repo = $this->composer->getRepositoryManager()->getLocalRepository();
+        $globalRepo = $this->globalComposer ? $this->globalComposer->getRepositoryManager()->getLocalRepository() : null;
+        $this->deactivateRepository($repo, false);
+        if ($globalRepo) {
+            $this->deactivateRepository($globalRepo, true);
+        }
+    }
+
     /**
      * Gets all currently active plugin instances
      *
@@ -130,6 +165,11 @@ class PluginManager
             return;
         }
 
+        if (!$this->isPluginAllowed($package->getName(), $isGlobalPlugin)) {
+            $this->io->writeError('Skipped loading "'.$package->getName() . '" '.($isGlobalPlugin ? '(installed globally) ' : '').'as it is not in config.allow-plugins', true, IOInterface::DEBUG);
+            return;
+        }
+
         if ($package->getType() === 'composer-plugin') {
             $requiresComposer = null;
             foreach ($package->getRequires() as $link) { /** @var Link $link */
@@ -154,7 +194,7 @@ class PluginManager
                 return;
             }
 
-            if ($package->getName() === 'symfony/flex' && preg_match('{^[0-9.]+$}', $package->getVersion()) && version_compare($package->getVersion(), '1.9.8', '<')) {
+            if ($package->getName() === 'symfony/flex' && Preg::isMatch('{^[0-9.]+$}', $package->getVersion()) && version_compare($package->getVersion(), '1.9.8', '<')) {
                 $this->io->writeError('<warning>The "' . $package->getName() . '" plugin '.($isGlobalPlugin ? '(installed globally) ' : '').'was skipped because it is not compatible with Composer 2+. Make sure to update it to version 1.9.8 or greater.</warning>');
 
                 return;
@@ -174,9 +214,20 @@ class PluginManager
         $classes = is_array($extra['class']) ? $extra['class'] : array($extra['class']);
 
         $localRepo = $this->composer->getRepositoryManager()->getLocalRepository();
-        $globalRepo = $this->globalComposer ? $this->globalComposer->getRepositoryManager()->getLocalRepository() : null;
+        $globalRepo = $this->globalComposer !== null ? $this->globalComposer->getRepositoryManager()->getLocalRepository() : null;
 
         $rootPackage = clone $this->composer->getPackage();
+
+        // clear files autoload rules from the root package as the root dependencies are not
+        // necessarily all present yet when booting this runtime autoloader
+        $rootPackageAutoloads = $rootPackage->getAutoload();
+        $rootPackageAutoloads['files'] = array();
+        $rootPackage->setAutoload($rootPackageAutoloads);
+        $rootPackageAutoloads = $rootPackage->getDevAutoload();
+        $rootPackageAutoloads['files'] = array();
+        $rootPackage->setDevAutoload($rootPackageAutoloads);
+        unset($rootPackageAutoloads);
+
         $rootPackageRepo = new RootPackageRepository($rootPackage);
         $installedRepo = new InstalledRepository(array($localRepo, $rootPackageRepo));
         if ($globalRepo) {
@@ -201,6 +252,16 @@ class PluginManager
         $classLoader = $generator->createLoader($map, $this->composer->getConfig()->get('vendor-dir'));
         $classLoader->register(false);
 
+        foreach ($map['files'] as $fileIdentifier => $file) {
+            // exclude laminas/laminas-zendframework-bridge:src/autoload.php as it breaks Composer in some conditions
+            // see https://github.com/composer/composer/issues/10349 and https://github.com/composer/composer/issues/10401
+            // this hack can be removed once this deprecated package stop being installed
+            if ($fileIdentifier === '7e9bd612cc444b3eed788ebbe46263a0') {
+                continue;
+            }
+            \Composer\Autoload\composerRequire($fileIdentifier, $file);
+        }
+
         foreach ($classes as $class) {
             if (class_exists($class, false)) {
                 $class = trim($class, '\\');
@@ -211,24 +272,30 @@ class PluginManager
                 if ($separatorPos) {
                     $className = substr($class, $separatorPos + 1);
                 }
-                $code = preg_replace('{^((?:final\s+)?(?:\s*))class\s+('.preg_quote($className).')}mi', '$1class $2_composer_tmp'.self::$classCounter, $code, 1);
+                $code = Preg::replace('{^((?:final\s+)?(?:\s*))class\s+('.preg_quote($className).')}mi', '$1class $2_composer_tmp'.self::$classCounter, $code, 1);
                 $code = strtr($code, array(
                     '__FILE__' => var_export($path, true),
                     '__DIR__' => var_export(dirname($path), true),
                     '__CLASS__' => var_export($class, true),
                 ));
-                $code = preg_replace('/^\s*<\?(php)?/i', '', $code, 1);
+                $code = Preg::replace('/^\s*<\?(php)?/i', '', $code, 1);
                 eval($code);
                 $class .= '_composer_tmp'.self::$classCounter;
                 self::$classCounter++;
             }
 
             if ($oldInstallerPlugin) {
+                if (!is_a($class, 'Composer\Installer\InstallerInterface', true)) {
+                    throw new \RuntimeException('Could not activate plugin "'.$package->getName().'" as "'.$class.'" does not implement Composer\Installer\InstallerInterface');
+                }
                 $this->io->writeError('<warning>Loading "'.$package->getName() . '" '.($isGlobalPlugin ? '(installed globally) ' : '').'which is a legacy composer-installer built for Composer 1.x, it is likely to cause issues as you are running Composer 2.x.</warning>');
                 $installer = new $class($this->io, $this->composer);
                 $this->composer->getInstallationManager()->addInstaller($installer);
                 $this->registeredPlugins[$package->getName()] = $installer;
             } elseif (class_exists($class)) {
+                if (!is_a($class, 'Composer\Plugin\PluginInterface', true)) {
+                    throw new \RuntimeException('Could not activate plugin "'.$package->getName().'" as "'.$class.'" does not implement Composer\Plugin\PluginInterface');
+                }
                 $plugin = new $class();
                 $this->addPlugin($plugin, $isGlobalPlugin, $package);
                 $this->registeredPlugins[$package->getName()] = $plugin;
@@ -256,20 +323,15 @@ class PluginManager
             return;
         }
 
-        $oldInstallerPlugin = ($package->getType() === 'composer-installer');
-
         if (!isset($this->registeredPlugins[$package->getName()])) {
             return;
         }
 
-        if ($oldInstallerPlugin) {
-            /** @var \Composer\Installer\InstallerInterface $installer */
-            $installer = $this->registeredPlugins[$package->getName()];
-            unset($this->registeredPlugins[$package->getName()]);
-            $this->composer->getInstallationManager()->removeInstaller($installer);
+        $plugin = $this->registeredPlugins[$package->getName()];
+        unset($this->registeredPlugins[$package->getName()]);
+        if ($plugin instanceof InstallerInterface) {
+            $this->composer->getInstallationManager()->removeInstaller($plugin);
         } else {
-            $plugin = $this->registeredPlugins[$package->getName()];
-            unset($this->registeredPlugins[$package->getName()]);
             $this->removePlugin($plugin);
         }
     }
@@ -292,16 +354,14 @@ class PluginManager
             return;
         }
 
-        $oldInstallerPlugin = ($package->getType() === 'composer-installer');
-
         if (!isset($this->registeredPlugins[$package->getName()])) {
             return;
         }
 
-        if ($oldInstallerPlugin) {
+        $plugin = $this->registeredPlugins[$package->getName()];
+        if ($plugin instanceof InstallerInterface) {
             $this->deactivatePackage($package);
         } else {
-            $plugin = $this->registeredPlugins[$package->getName()];
             unset($this->registeredPlugins[$package->getName()]);
             $this->removePlugin($plugin);
             $this->uninstallPlugin($plugin);
@@ -333,6 +393,13 @@ class PluginManager
      */
     public function addPlugin(PluginInterface $plugin, $isGlobalPlugin = false, PackageInterface $sourcePackage = null)
     {
+        if ($sourcePackage === null) {
+            trigger_error('Calling PluginManager::addPlugin without $sourcePackage is deprecated, if you are using this please get in touch with us to explain the use case', E_USER_DEPRECATED);
+        } elseif (!$this->isPluginAllowed($sourcePackage->getName(), $isGlobalPlugin)) {
+            $this->io->writeError('Skipped loading "'.get_class($plugin).' from '.$sourcePackage->getName() . '" '.($isGlobalPlugin ? '(installed globally) ' : '').' as it is not in config.allow-plugins', true, IOInterface::DEBUG);
+            return;
+        }
+
         $details = array();
         if ($sourcePackage) {
             $details[] = 'from '.$sourcePackage->getName();
@@ -424,6 +491,34 @@ class PluginManager
         }
     }
 
+    /**
+     * Deactivate all plugins and installers from a repository
+     *
+     * If a plugin requires another plugin, the required one will be deactivated last
+     *
+     * @param RepositoryInterface $repo Repository to scan for plugins to install
+     * @param bool                $isGlobalRepo
+     *
+     * @return void
+     */
+    private function deactivateRepository(RepositoryInterface $repo, $isGlobalRepo)
+    {
+        $packages = $repo->getPackages();
+        $sortedPackages = array_reverse(PackageSorter::sortPackages($packages));
+
+        foreach ($sortedPackages as $package) {
+            if (!($package instanceof CompletePackage)) {
+                continue;
+            }
+            if ('composer-plugin' === $package->getType()) {
+                $this->deactivatePackage($package);
+            // Backward compatibility
+            } elseif ('composer-installer' === $package->getType()) {
+                $this->deactivatePackage($package);
+            }
+        }
+    }
+
     /**
      * Recursively generates a map of package names to packages for all deps
      *
@@ -545,4 +640,113 @@ class PluginManager
 
         return $capabilities;
     }
+
+    /**
+     * @param  array<string, bool>|bool|null $allowPluginsConfig
+     * @return array<non-empty-string, bool>|null
+     */
+    private function parseAllowedPlugins($allowPluginsConfig)
+    {
+        if (null === $allowPluginsConfig) {
+            return null;
+        }
+
+        if (true === $allowPluginsConfig) {
+            return array('{}' => true);
+        }
+
+        if (false === $allowPluginsConfig) {
+            return array('{^$}D' => false);
+        }
+
+        $rules = array();
+        foreach ($allowPluginsConfig as $pattern => $allow) {
+            $rules[BasePackage::packageNameToRegexp($pattern)] = $allow;
+        }
+
+        return $rules;
+    }
+
+    /**
+     * @param string $package
+     * @param bool $isGlobalPlugin
+     * @return bool
+     */
+    private function isPluginAllowed($package, $isGlobalPlugin)
+    {
+        static $warned = array();
+        $rules = $isGlobalPlugin ? $this->allowGlobalPluginRules : $this->allowPluginRules;
+
+        if ($rules === null) {
+            if (!$this->io->isInteractive()) {
+                if (!isset($warned['all'])) {
+                    $this->io->writeError('<warning>For additional security you should declare the allow-plugins config with a list of packages names that are allowed to run code. See https://getcomposer.org/allow-plugins</warning>');
+                    $this->io->writeError('<warning>You have until July 2022 to add the setting. Composer will then switch the default behavior to disallow all plugins.</warning>');
+                    $warned['all'] = true;
+                }
+
+                // if no config is defined we allow all plugins for BC
+                return true;
+            }
+
+            // keep going and prompt the user
+            $rules = array();
+        }
+
+        foreach ($rules as $pattern => $allow) {
+            if (Preg::isMatch($pattern, $package)) {
+                return $allow === true;
+            }
+        }
+
+        if ($package === 'composer/package-versions-deprecated') {
+            return false;
+        }
+
+        if (!isset($warned[$package])) {
+            if ($this->io->isInteractive()) {
+                $composer = $isGlobalPlugin && $this->globalComposer !== null ? $this->globalComposer : $this->composer;
+
+                $this->io->writeError('<warning>'.$package.($isGlobalPlugin ? ' (installed globally)' : '').' contains a Composer plugin which is currently not in your allow-plugins config. See https://getcomposer.org/allow-plugins</warning>');
+                while (true) {
+                    switch ($answer = $this->io->ask('Do you trust "<info>'.$package.'</info>" to execute code and wish to enable it now? (writes "allow-plugins" to composer.json) [<comment>y,n,d,?</comment>] ', '?')) {
+                        case 'y':
+                        case 'n':
+                        case 'd':
+                            $allow = $answer === 'y';
+
+                            // persist answer in current rules to avoid prompting again if the package gets reloaded
+                            if ($isGlobalPlugin) {
+                                $this->allowGlobalPluginRules[BasePackage::packageNameToRegexp($package)] = $allow;
+                            } else {
+                                $this->allowPluginRules[BasePackage::packageNameToRegexp($package)] = $allow;
+                            }
+
+                            // persist answer in composer.json if it wasn't simply discarded
+                            if ($answer === 'y' || $answer === 'n') {
+                                $composer->getConfig()->getConfigSource()->addConfigSetting('allow-plugins.'.$package, $allow);
+                            }
+
+                            return $allow;
+
+                        case '?':
+                        default:
+                            $this->io->writeError(array(
+                                'y - add package to allow-plugins in composer.json and let it run immediately',
+                                'n - add package (as disallowed) to allow-plugins in composer.json to suppress further prompts',
+                                'd - discard this, do not change composer.json and do not allow the plugin to run',
+                                '? - print help'
+                            ));
+                            break;
+                    }
+                }
+            } else {
+                $this->io->writeError('<warning>'.$package.($isGlobalPlugin ? ' (installed globally)' : '').' contains a Composer plugin which is blocked by your allow-plugins config. You may add it to the list if you consider it safe. See https://getcomposer.org/allow-plugins</warning>');
+                $this->io->writeError('<warning>You can run "composer '.($isGlobalPlugin ? 'global ' : '').'config --no-plugins allow-plugins.'.$package.' [true|false]" to enable it (true) or keep it disabled and suppress this warning (false)</warning>');
+            }
+            $warned[$package] = true;
+        }
+
+        return false;
+    }
 }
