@@ -47,6 +47,11 @@ class PreFileDownloadEvent extends Event
      */
     private $context;
 
+    /**
+     * @var mixed[]
+     */
+    private $transportOptions = array();
+
     /**
      * Constructor.
      *
@@ -87,6 +92,8 @@ class PreFileDownloadEvent extends Event
      * Sets the processed URL that will be downloaded.
      *
      * @param string $processedUrl New processed URL
+     *
+     * @return void
      */
     public function setProcessedUrl($processedUrl)
     {
@@ -107,6 +114,8 @@ class PreFileDownloadEvent extends Event
      * Sets a custom package cache key for this download.
      *
      * @param string|null $customCacheKey New cache key
+     *
+     * @return void
      */
     public function setCustomCacheKey($customCacheKey)
     {
@@ -127,6 +136,7 @@ class PreFileDownloadEvent extends Event
      * Returns the context of this download, if any.
      *
      * If this download is of type package, the package object is returned.
+     * If the type is metadata, an array{repository: RepositoryInterface} is returned.
      *
      * @return mixed
      */
@@ -134,4 +144,30 @@ class PreFileDownloadEvent extends Event
     {
         return $this->context;
     }
+
+    /**
+     * Returns transport options for the download.
+     *
+     * Only available for events with type metadata, for packages set the transport options on the package itself.
+     *
+     * @return mixed[]
+     */
+    public function getTransportOptions()
+    {
+        return $this->transportOptions;
+    }
+
+    /**
+     * Sets transport options for the download.
+     *
+     * Only available for events with type metadata, for packages set the transport options on the package itself.
+     *
+     * @param mixed[] $options
+     *
+     * @return void
+     */
+    public function setTransportOptions(array $options)
+    {
+        $this->transportOptions = $options;
+    }
 }
