@@ -20,6 +20,7 @@ use Composer\Package\PackageInterface;
 use Composer\Package\CompletePackageInterface;
 use Composer\Package\Version\VersionParser;
 use Composer\Package\Version\StabilityFilter;
+use Composer\Pcre\Preg;
 use Composer\Semver\Constraint\ConstraintInterface;
 use Composer\Semver\Constraint\Constraint;
 
@@ -147,28 +148,43 @@ class ArrayRepository implements RepositoryInterface
      */
     public function search($query, $mode = 0, $type = null)
     {
-        $regex = '{(?:'.implode('|', preg_split('{\s+}', $query)).')}i';
+        if ($mode === self::SEARCH_FULLTEXT) {
+            $regex = '{(?:'.implode('|', Preg::split('{\s+}', preg_quote($query))).')}i';
+        } else {
+            // vendor/name searches expect the caller to have preg_quoted the query
+            $regex = '{(?:'.implode('|', Preg::split('{\s+}', $query)).')}i';
+        }
 
         $matches = array();
         foreach ($this->getPackages() as $package) {
             $name = $package->getName();
+            if ($mode === self::SEARCH_VENDOR) {
+                list($name) = explode('/', $name);
+            }
             if (isset($matches[$name])) {
                 continue;
             }
-            if (preg_match($regex, $name)
-                || ($mode === self::SEARCH_FULLTEXT && $package instanceof CompletePackageInterface && preg_match($regex, implode(' ', (array) $package->getKeywords()) . ' ' . $package->getDescription()))
+            if (null !== $type && $package->getType() !== $type) {
+                continue;
+            }
+
+            if (Preg::isMatch($regex, $name)
+                || ($mode === self::SEARCH_FULLTEXT && $package instanceof CompletePackageInterface && Preg::isMatch($regex, implode(' ', (array) $package->getKeywords()) . ' ' . $package->getDescription()))
             ) {
-                if (null !== $type && $package->getType() !== $type) {
-                    continue;
-                }
+                if ($mode === self::SEARCH_VENDOR) {
+                    $matches[$name] = array(
+                        'name' => $name,
+                        'description' => null,
+                    );
+                } else {
+                    $matches[$name] = array(
+                        'name' => $package->getPrettyName(),
+                        'description' => $package instanceof CompletePackageInterface ? $package->getDescription() : null,
+                    );
 
-                $matches[$name] = array(
-                    'name' => $package->getPrettyName(),
-                    'description' => $package instanceof CompletePackageInterface ? $package->getDescription() : null,
-                );
-
-                if ($package instanceof CompletePackageInterface && $package->isAbandoned()) {
-                    $matches[$name]['abandoned'] = $package->getReplacementPackage() ?: true;
+                    if ($package instanceof CompletePackageInterface && $package->isAbandoned()) {
+                        $matches[$name]['abandoned'] = $package->getReplacementPackage() ?: true;
+                    }
                 }
             }
         }
@@ -194,12 +210,13 @@ class ArrayRepository implements RepositoryInterface
     /**
      * Adds a new package to the repository
      *
-     * @param PackageInterface $package
-     *
      * @return void
      */
     public function addPackage(PackageInterface $package)
     {
+        if (!$package instanceof BasePackage) {
+            throw new \InvalidArgumentException('Only subclasses of BasePackage are supported');
+        }
         if (null === $this->packages) {
             $this->initialize();
         }
