@@ -21,6 +21,11 @@ namespace Composer\Repository;
  */
 interface InstalledRepositoryInterface extends WritableRepositoryInterface
 {
+    /**
+     * @return bool|null true if dev requirements were installed, false if --no-dev was used, null if yet unknown
+     */
+    public function getDevMode();
+
     /**
      * @return bool true if packages were never installed in this repository
      */
