@@ -14,9 +14,11 @@ namespace Composer\Repository;
 
 use Composer\Composer;
 use Composer\Package\CompletePackage;
+use Composer\Package\CompletePackageInterface;
 use Composer\Package\Link;
 use Composer\Package\PackageInterface;
 use Composer\Package\Version\VersionParser;
+use Composer\Pcre\Preg;
 use Composer\Platform\HhvmDetector;
 use Composer\Platform\Runtime;
 use Composer\Platform\Version;
@@ -30,7 +32,12 @@ use Composer\XdebugHandler\XdebugHandler;
  */
 class PlatformRepository extends ArrayRepository
 {
-    const PLATFORM_PACKAGE_REGEX = '{^(?:php(?:-64bit|-ipv6|-zts|-debug)?|hhvm|(?:ext|lib)-[a-z0-9](?:[_.-]?[a-z0-9]+)*|composer-(?:plugin|runtime)-api)$}iD';
+    const PLATFORM_PACKAGE_REGEX = '{^(?:php(?:-64bit|-ipv6|-zts|-debug)?|hhvm|(?:ext|lib)-[a-z0-9](?:[_.-]?[a-z0-9]+)*|composer(?:-(?:plugin|runtime)-api)?)$}iD';
+
+    /**
+     * @var ?string
+     */
+    private static $lastSeenPlatformPhp = null;
 
     /**
      * @var VersionParser
@@ -40,20 +47,38 @@ class PlatformRepository extends ArrayRepository
     /**
      * Defines overrides so that the platform can be mocked
      *
-     * Should be an array of package name => version number mappings
+     * Keyed by package name (lowercased)
      *
-     * @var array
+     * @var array<string, array{name: string, version: string|false}>
      */
     private $overrides = array();
 
+    /**
+     * Stores which packages have been disabled and their actual version
+     *
+     * @var array<string, CompletePackageInterface>
+     */
+    private $disabledPackages = array();
+
+    /** @var Runtime */
     private $runtime;
+    /** @var HhvmDetector */
     private $hhvmDetector;
 
+    /**
+     * @param array<string, string|false> $overrides
+     */
     public function __construct(array $packages = array(), array $overrides = array(), Runtime $runtime = null, HhvmDetector $hhvmDetector = null)
     {
         $this->runtime = $runtime ?: new Runtime();
         $this->hhvmDetector = $hhvmDetector ?: new HhvmDetector();
         foreach ($overrides as $name => $version) {
+            if (!is_string($version) && false !== $version) { // @phpstan-ignore-line
+                throw new \UnexpectedValueException('config.platform.'.$name.' should be a string or false, but got '.gettype($version).' '.var_export($version, true));
+            }
+            if ($name === 'php' && $version === false) {
+                throw new \UnexpectedValueException('config.platform.'.$name.' cannot be set to false as you cannot disable php entirely.');
+            }
             $this->overrides[strtolower($name)] = array('name' => $name, 'version' => $version);
         }
         parent::__construct($packages);
@@ -64,6 +89,23 @@ class PlatformRepository extends ArrayRepository
         return 'platform repo';
     }
 
+    /**
+     * @param  string  $name
+     * @return boolean
+     */
+    public function isPlatformPackageDisabled($name)
+    {
+        return isset($this->disabledPackages[$name]);
+    }
+
+    /**
+     * @return array<string, CompletePackageInterface>
+     */
+    public function getDisabledPackages()
+    {
+        return $this->disabledPackages;
+    }
+
     protected function initialize()
     {
         parent::initialize();
@@ -78,9 +120,17 @@ class PlatformRepository extends ArrayRepository
                 throw new \InvalidArgumentException('Invalid platform package name in config.platform: '.$override['name']);
             }
 
-            $this->addOverriddenPackage($override);
+            if ($override['version'] !== false) {
+                $this->addOverriddenPackage($override);
+            }
         }
 
+        $prettyVersion = Composer::getVersion();
+        $version = $this->versionParser->normalize($prettyVersion);
+        $composer = new CompletePackage('composer', $version, $prettyVersion);
+        $composer->setDescription('Composer package');
+        $this->addPackage($composer);
+
         $prettyVersion = PluginInterface::PLUGIN_API_VERSION;
         $version = $this->versionParser->normalize($prettyVersion);
         $composerPluginApi = new CompletePackage('composer-plugin-api', $version, $prettyVersion);
@@ -97,7 +147,7 @@ class PlatformRepository extends ArrayRepository
             $prettyVersion = $this->runtime->getConstant('PHP_VERSION');
             $version = $this->versionParser->normalize($prettyVersion);
         } catch (\UnexpectedValueException $e) {
-            $prettyVersion = preg_replace('#^([^~+-]+).*$#', '$1', $this->runtime->getConstant('PHP_VERSION'));
+            $prettyVersion = Preg::replace('#^([^~+-]+).*$#', '$1', $this->runtime->getConstant('PHP_VERSION'));
             $version = $this->versionParser->normalize($prettyVersion);
         }
 
@@ -156,12 +206,12 @@ class PlatformRepository extends ArrayRepository
                     $info = $this->runtime->getExtensionInfo($name);
 
                     // librabbitmq version => 0.9.0
-                    if (preg_match('/^librabbitmq version => (?<version>.+)$/im', $info, $librabbitmqMatches)) {
+                    if (Preg::isMatch('/^librabbitmq version => (?<version>.+)$/im', $info, $librabbitmqMatches)) {
                         $this->addLibrary($name.'-librabbitmq', $librabbitmqMatches['version'], 'AMQP librabbitmq version');
                     }
 
                     // AMQP protocol version => 0-9-1
-                    if (preg_match('/^AMQP protocol version => (?<version>.+)$/im', $info, $protocolMatches)) {
+                    if (Preg::isMatch('/^AMQP protocol version => (?<version>.+)$/im', $info, $protocolMatches)) {
                         $this->addLibrary($name.'-protocol', str_replace('-', '.', $protocolMatches['version']), 'AMQP protocol version');
                     }
                     break;
@@ -170,7 +220,7 @@ class PlatformRepository extends ArrayRepository
                     $info = $this->runtime->getExtensionInfo($name);
 
                     // BZip2 Version => 1.0.6, 6-Sept-2010
-                    if (preg_match('/^BZip2 Version => (?<version>.*),/im', $info, $matches)) {
+                    if (Preg::isMatch('/^BZip2 Version => (?<version>.*),/im', $info, $matches)) {
                         $this->addLibrary($name, $matches['version']);
                     }
                     break;
@@ -182,7 +232,7 @@ class PlatformRepository extends ArrayRepository
                     $info = $this->runtime->getExtensionInfo($name);
 
                     // SSL Version => OpenSSL/1.0.1t
-                    if (preg_match('{^SSL Version => (?<library>[^/]+)/(?<version>.+)$}im', $info, $sslMatches)) {
+                    if (Preg::isMatch('{^SSL Version => (?<library>[^/]+)/(?<version>.+)$}im', $info, $sslMatches)) {
                         $library = strtolower($sslMatches['library']);
                         if ($library === 'openssl') {
                             $parsedVersion = Version::parseOpenssl($sslMatches['version'], $isFips);
@@ -193,12 +243,12 @@ class PlatformRepository extends ArrayRepository
                     }
 
                     // libSSH Version => libssh2/1.4.3
-                    if (preg_match('{^libSSH Version => (?<library>[^/]+)/(?<version>.+?)(?:/.*)?$}im', $info, $sshMatches)) {
+                    if (Preg::isMatch('{^libSSH Version => (?<library>[^/]+)/(?<version>.+?)(?:/.*)?$}im', $info, $sshMatches)) {
                         $this->addLibrary($name.'-'.strtolower($sshMatches['library']), $sshMatches['version'], 'curl '.$sshMatches['library'].' version');
                     }
 
                     // ZLib Version => 1.2.8
-                    if (preg_match('{^ZLib Version => (?<version>.+)$}im', $info, $zlibMatches)) {
+                    if (Preg::isMatch('{^ZLib Version => (?<version>.+)$}im', $info, $zlibMatches)) {
                         $this->addLibrary($name.'-zlib', $zlibMatches['version'], 'curl zlib version');
                     }
                     break;
@@ -207,14 +257,14 @@ class PlatformRepository extends ArrayRepository
                     $info = $this->runtime->getExtensionInfo($name);
 
                     // timelib version => 2018.03
-                    if (preg_match('/^timelib version => (?<version>.+)$/im', $info, $timelibMatches)) {
+                    if (Preg::isMatch('/^timelib version => (?<version>.+)$/im', $info, $timelibMatches)) {
                         $this->addLibrary($name.'-timelib', $timelibMatches['version'], 'date timelib version');
                     }
 
                     // Timezone Database => internal
-                    if (preg_match('/^Timezone Database => (?<source>internal|external)$/im', $info, $zoneinfoSourceMatches)) {
+                    if (Preg::isMatch('/^Timezone Database => (?<source>internal|external)$/im', $info, $zoneinfoSourceMatches)) {
                         $external = $zoneinfoSourceMatches['source'] === 'external';
-                        if (preg_match('/^"Olson" Timezone Database Version => (?<version>.+?)(\.system)?$/im', $info, $zoneinfoMatches)) {
+                        if (Preg::isMatch('/^"Olson" Timezone Database Version => (?<version>.+?)(\.system)?$/im', $info, $zoneinfoMatches)) {
                             // If the timezonedb is provided by ext/timezonedb, register that version as a replacement
                             if ($external && in_array('timezonedb', $loadedExtensions, true)) {
                                 $this->addLibrary('timezonedb-zoneinfo', $zoneinfoMatches['version'], 'zoneinfo ("Olson") database for date (replaced by timezonedb)', array($name.'-zoneinfo'));
@@ -229,7 +279,7 @@ class PlatformRepository extends ArrayRepository
                     $info = $this->runtime->getExtensionInfo($name);
 
                     // libmagic => 537
-                    if (preg_match('/^libmagic => (?<version>.+)$/im', $info, $magicMatches)) {
+                    if (Preg::isMatch('/^libmagic => (?<version>.+)$/im', $info, $magicMatches)) {
                         $this->addLibrary($name.'-libmagic', $magicMatches['version'], 'fileinfo libmagic version');
                     }
                     break;
@@ -239,19 +289,19 @@ class PlatformRepository extends ArrayRepository
 
                     $info = $this->runtime->getExtensionInfo($name);
 
-                    if (preg_match('/^libJPEG Version => (?<version>.+?)(?: compatible)?$/im', $info, $libjpegMatches)) {
+                    if (Preg::isMatch('/^libJPEG Version => (?<version>.+?)(?: compatible)?$/im', $info, $libjpegMatches)) {
                         $this->addLibrary($name.'-libjpeg', Version::parseLibjpeg($libjpegMatches['version']), 'libjpeg version for gd');
                     }
 
-                    if (preg_match('/^libPNG Version => (?<version>.+)$/im', $info, $libpngMatches)) {
+                    if (Preg::isMatch('/^libPNG Version => (?<version>.+)$/im', $info, $libpngMatches)) {
                         $this->addLibrary($name.'-libpng', $libpngMatches['version'], 'libpng version for gd');
                     }
 
-                    if (preg_match('/^FreeType Version => (?<version>.+)$/im', $info, $freetypeMatches)) {
+                    if (Preg::isMatch('/^FreeType Version => (?<version>.+)$/im', $info, $freetypeMatches)) {
                         $this->addLibrary($name.'-freetype', $freetypeMatches['version'], 'freetype version for gd');
                     }
 
-                    if (preg_match('/^libXpm Version => (?<versionId>\d+)$/im', $info, $libxpmMatches)) {
+                    if (Preg::isMatch('/^libXpm Version => (?<versionId>\d+)$/im', $info, $libxpmMatches)) {
                         $this->addLibrary($name.'-libxpm', Version::convertLibxpmVersionId($libxpmMatches['versionId']), 'libxpm version for gd');
                     }
 
@@ -272,12 +322,12 @@ class PlatformRepository extends ArrayRepository
                     // Truthy check is for testing only so we can make the condition fail
                     if ($this->runtime->hasConstant('INTL_ICU_VERSION')) {
                         $this->addLibrary('icu', $this->runtime->getConstant('INTL_ICU_VERSION'), $description);
-                    } elseif (preg_match('/^ICU version => (?<version>.+)$/im', $info, $matches)) {
+                    } elseif (Preg::isMatch('/^ICU version => (?<version>.+)$/im', $info, $matches)) {
                         $this->addLibrary('icu', $matches['version'], $description);
                     }
 
                     // ICU TZData version => 2019c
-                    if (preg_match('/^ICU TZData version => (?<version>.*)$/im', $info, $zoneinfoMatches)) {
+                    if (Preg::isMatch('/^ICU TZData version => (?<version>.*)$/im', $info, $zoneinfoMatches)) {
                         $this->addLibrary('icu-zoneinfo', Version::parseZoneinfoVersion($zoneinfoMatches['version']), 'zoneinfo ("Olson") database for icu');
                     }
 
@@ -296,7 +346,7 @@ class PlatformRepository extends ArrayRepository
                     $imageMagickVersion = $this->runtime->construct('Imagick')->getVersion();
                     // 6.x: ImageMagick 6.2.9 08/24/06 Q16 http://www.imagemagick.org
                     // 7.x: ImageMagick 7.0.8-34 Q16 x86_64 2019-03-23 https://imagemagick.org
-                    preg_match('/^ImageMagick (?<version>[\d.]+)(?:-(?<patch>\d+))?/', $imageMagickVersion['versionString'], $matches);
+                    Preg::match('/^ImageMagick (?<version>[\d.]+)(?:-(?<patch>\d+))?/', $imageMagickVersion['versionString'], $matches);
                     $version = $matches['version'];
                     if (isset($matches['patch'])) {
                         $version .= '.'.$matches['patch'];
@@ -308,7 +358,7 @@ class PlatformRepository extends ArrayRepository
                 case 'ldap':
                     $info = $this->runtime->getExtensionInfo($name);
 
-                    if (preg_match('/^Vendor Version => (?<versionId>\d+)$/im', $info, $matches) && preg_match('/^Vendor Name => (?<vendor>.+)$/im', $info, $vendorMatches)) {
+                    if (Preg::isMatch('/^Vendor Version => (?<versionId>\d+)$/im', $info, $matches) && Preg::isMatch('/^Vendor Name => (?<vendor>.+)$/im', $info, $vendorMatches)) {
                         $this->addLibrary($name.'-'.strtolower($vendorMatches['vendor']), Version::convertOpenldapVersionId($matches['versionId']), $vendorMatches['vendor'].' version of ldap');
                     }
                     break;
@@ -326,7 +376,7 @@ class PlatformRepository extends ArrayRepository
                     $info = $this->runtime->getExtensionInfo($name);
 
                     // libmbfl version => 1.3.2
-                    if (preg_match('/^libmbfl version => (?<version>.+)$/im', $info, $libmbflMatches)) {
+                    if (Preg::isMatch('/^libmbfl version => (?<version>.+)$/im', $info, $libmbflMatches)) {
                         $this->addLibrary($name.'-libmbfl', $libmbflMatches['version'], 'mbstring libmbfl version');
                     }
 
@@ -335,7 +385,7 @@ class PlatformRepository extends ArrayRepository
 
                     // Multibyte regex (oniguruma) version => 5.9.5
                     // oniguruma version => 6.9.0
-                    } elseif (preg_match('/^(?:oniguruma|Multibyte regex \(oniguruma\)) version => (?<version>.+)$/im', $info, $onigurumaMatches)) {
+                    } elseif (Preg::isMatch('/^(?:oniguruma|Multibyte regex \(oniguruma\)) version => (?<version>.+)$/im', $info, $onigurumaMatches)) {
                         $this->addLibrary($name.'-oniguruma', $onigurumaMatches['version'], 'mbstring oniguruma version');
                     }
 
@@ -345,26 +395,26 @@ class PlatformRepository extends ArrayRepository
                     $info = $this->runtime->getExtensionInfo($name);
 
                     // libmemcached version => 1.0.18
-                    if (preg_match('/^libmemcached version => (?<version>.+)$/im', $info, $matches)) {
+                    if (Preg::isMatch('/^libmemcached version => (?<version>.+)$/im', $info, $matches)) {
                         $this->addLibrary($name.'-libmemcached', $matches['version'], 'libmemcached version');
                     }
                     break;
 
                 case 'openssl':
                     // OpenSSL 1.1.1g  21 Apr 2020
-                    if (preg_match('{^(?:OpenSSL|LibreSSL)?\s*(?<version>\S+)}i', $this->runtime->getConstant('OPENSSL_VERSION_TEXT'), $matches)) {
+                    if (Preg::isMatch('{^(?:OpenSSL|LibreSSL)?\s*(?<version>\S+)}i', $this->runtime->getConstant('OPENSSL_VERSION_TEXT'), $matches)) {
                         $parsedVersion = Version::parseOpenssl($matches['version'], $isFips);
                         $this->addLibrary($name.($isFips ? '-fips' : ''), $parsedVersion, $this->runtime->getConstant('OPENSSL_VERSION_TEXT'), array(), $isFips ? array($name) : array());
                     }
                     break;
 
                 case 'pcre':
-                    $this->addLibrary($name, preg_replace('{^(\S+).*}', '$1', $this->runtime->getConstant('PCRE_VERSION')));
+                    $this->addLibrary($name, Preg::replace('{^(\S+).*}', '$1', $this->runtime->getConstant('PCRE_VERSION')));
 
                     $info = $this->runtime->getExtensionInfo($name);
 
                     // PCRE Unicode Version => 12.1.0
-                    if (preg_match('/^PCRE Unicode Version => (?<version>.+)$/im', $info, $pcreUnicodeMatches)) {
+                    if (Preg::isMatch('/^PCRE Unicode Version => (?<version>.+)$/im', $info, $pcreUnicodeMatches)) {
                         $this->addLibrary($name.'-unicode', $pcreUnicodeMatches['version'], 'PCRE Unicode version support');
                     }
 
@@ -374,7 +424,7 @@ class PlatformRepository extends ArrayRepository
                 case 'pdo_mysql':
                     $info = $this->runtime->getExtensionInfo($name);
 
-                    if (preg_match('/^(?:Client API version|Version) => mysqlnd (?<version>.+?) /mi', $info, $matches)) {
+                    if (Preg::isMatch('/^(?:Client API version|Version) => mysqlnd (?<version>.+?) /mi', $info, $matches)) {
                         $this->addLibrary($name.'-mysqlnd', $matches['version'], 'mysqlnd library version for '.$name);
                     }
                     break;
@@ -382,11 +432,11 @@ class PlatformRepository extends ArrayRepository
                 case 'mongodb':
                     $info = $this->runtime->getExtensionInfo($name);
 
-                    if (preg_match('/^libmongoc bundled version => (?<version>.+)$/im', $info, $libmongocMatches)) {
+                    if (Preg::isMatch('/^libmongoc bundled version => (?<version>.+)$/im', $info, $libmongocMatches)) {
                         $this->addLibrary($name.'-libmongoc', $libmongocMatches['version'], 'libmongoc version of mongodb');
                     }
 
-                    if (preg_match('/^libbson bundled version => (?<version>.+)$/im', $info, $libbsonMatches)) {
+                    if (Preg::isMatch('/^libbson bundled version => (?<version>.+)$/im', $info, $libbsonMatches)) {
                         $this->addLibrary($name.'-libbson', $libbsonMatches['version'], 'libbson version of mongodb');
                     }
                     break;
@@ -395,21 +445,23 @@ class PlatformRepository extends ArrayRepository
                 case 'pdo_pgsql':
                     $info = $this->runtime->getExtensionInfo($name);
 
-                    if (preg_match('/^PostgreSQL\(libpq\) Version => (?<version>.*)$/im', $info, $matches)) {
+                    if (Preg::isMatch('/^PostgreSQL\(libpq\) Version => (?<version>.*)$/im', $info, $matches)) {
                         $this->addLibrary($name.'-libpq', $matches['version'], 'libpq for '.$name);
                     }
                     break;
 
                 case 'libsodium':
                 case 'sodium':
-                    $this->addLibrary('libsodium', $this->runtime->getConstant('SODIUM_LIBRARY_VERSION'));
+                    if ($this->runtime->hasConstant('SODIUM_LIBRARY_VERSION')) {
+                        $this->addLibrary('libsodium', $this->runtime->getConstant('SODIUM_LIBRARY_VERSION'));
+                    }
                     break;
 
                 case 'sqlite3':
                 case 'pdo_sqlite':
                     $info = $this->runtime->getExtensionInfo($name);
 
-                    if (preg_match('/^SQLite Library => (?<version>.+)$/im', $info, $matches)) {
+                    if (Preg::isMatch('/^SQLite Library => (?<version>.+)$/im', $info, $matches)) {
                         $this->addLibrary($name.'-sqlite', $matches['version']);
                     }
                     break;
@@ -417,7 +469,7 @@ class PlatformRepository extends ArrayRepository
                 case 'ssh2':
                     $info = $this->runtime->getExtensionInfo($name);
 
-                    if (preg_match('/^libssh2 version => (?<version>.+)$/im', $info, $matches)) {
+                    if (Preg::isMatch('/^libssh2 version => (?<version>.+)$/im', $info, $matches)) {
                         $this->addLibrary($name.'-libssh2', $matches['version']);
                     }
                     break;
@@ -426,7 +478,7 @@ class PlatformRepository extends ArrayRepository
                     $this->addLibrary('libxslt', $this->runtime->getConstant('LIBXSLT_DOTTED_VERSION'), null, array('xsl'));
 
                     $info = $this->runtime->getExtensionInfo('xsl');
-                    if (preg_match('/^libxslt compiled against libxml Version => (?<version>.+)$/im', $info, $matches)) {
+                    if (Preg::isMatch('/^libxslt compiled against libxml Version => (?<version>.+)$/im', $info, $matches)) {
                         $this->addLibrary('libxslt-libxml', $matches['version'], 'libxml version libxslt is compiled against');
                     }
                     break;
@@ -434,7 +486,7 @@ class PlatformRepository extends ArrayRepository
                 case 'yaml':
                     $info = $this->runtime->getExtensionInfo('yaml');
 
-                    if (preg_match('/^LibYAML Version => (?<version>.+)$/im', $info, $matches)) {
+                    if (Preg::isMatch('/^LibYAML Version => (?<version>.+)$/im', $info, $matches)) {
                         $this->addLibrary($name.'-libyaml', $matches['version'], 'libyaml version of yaml');
                     }
                     break;
@@ -450,7 +502,7 @@ class PlatformRepository extends ArrayRepository
                         $this->addLibrary($name, $this->runtime->getConstant('ZLIB_VERSION'));
 
                     // Linked Version => 1.2.8
-                    } elseif (preg_match('/^Linked Version => (?<version>.+)$/im', $this->runtime->getExtensionInfo($name), $matches)) {
+                    } elseif (Preg::isMatch('/^Linked Version => (?<version>.+)$/im', $this->runtime->getExtensionInfo($name), $matches)) {
                         $this->addLibrary($name, $matches['version']);
                     }
                     break;
@@ -466,7 +518,7 @@ class PlatformRepository extends ArrayRepository
                 $prettyVersion = $hhvmVersion;
                 $version = $this->versionParser->normalize($prettyVersion);
             } catch (\UnexpectedValueException $e) {
-                $prettyVersion = preg_replace('#^([^~+-]+).*$#', '$1', $hhvmVersion);
+                $prettyVersion = Preg::replace('#^([^~+-]+).*$#', '$1', $hhvmVersion);
                 $version = $this->versionParser->normalize($prettyVersion);
             }
 
@@ -477,25 +529,41 @@ class PlatformRepository extends ArrayRepository
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function addPackage(PackageInterface $package)
     {
+        if (!$package instanceof CompletePackage) {
+            throw new \UnexpectedValueException('Expected CompletePackage but got '.get_class($package));
+        }
+
         // Skip if overridden
         if (isset($this->overrides[$package->getName()])) {
+            if ($this->overrides[$package->getName()]['version'] === false) {
+                $this->addDisabledPackage($package);
+                return;
+            }
+
             $overrider = $this->findPackage($package->getName(), '*');
             if ($package->getVersion() === $overrider->getVersion()) {
                 $actualText = 'same as actual';
             } else {
                 $actualText = 'actual: '.$package->getPrettyVersion();
             }
-            $overrider->setDescription($overrider->getDescription().', '.$actualText);
+            if ($overrider instanceof CompletePackageInterface) {
+                $overrider->setDescription($overrider->getDescription().', '.$actualText);
+            }
 
             return;
         }
 
         // Skip if PHP is overridden and we are adding a php-* package
         if (isset($this->overrides['php']) && 0 === strpos($package->getName(), 'php-')) {
+            if (isset($this->overrides[$package->getName()]) && $this->overrides[$package->getName()]['version'] === false) {
+                $this->addDisabledPackage($package);
+                return;
+            }
+
             $overrider = $this->addOverriddenPackage($this->overrides['php'], $package->getPrettyName());
             if ($package->getVersion() === $overrider->getVersion()) {
                 $actualText = 'same as actual';
@@ -510,6 +578,12 @@ class PlatformRepository extends ArrayRepository
         parent::addPackage($package);
     }
 
+    /**
+     * @param array{version: string, name: string} $override
+     * @param string|null $name
+     *
+     * @return CompletePackage
+     */
     private function addOverriddenPackage(array $override, $name = null)
     {
         $version = $this->versionParser->normalize($override['version']);
@@ -518,14 +592,31 @@ class PlatformRepository extends ArrayRepository
         $package->setExtra(array('config.platform' => true));
         parent::addPackage($package);
 
+        if ($package->getName() === 'php') {
+            self::$lastSeenPlatformPhp = implode('.', array_slice(explode('.', $package->getVersion()), 0, 3));
+        }
+
         return $package;
     }
 
+    /**
+     * @return void
+     */
+    private function addDisabledPackage(CompletePackage $package)
+    {
+        $package->setDescription($package->getDescription().'. <warning>Package disabled via config.platform</warning>');
+        $package->setExtra(array('config.platform' => true));
+
+        $this->disabledPackages[$package->getName()] = $package;
+    }
+
     /**
      * Parses the version and adds a new package to the repository
      *
      * @param string      $name
      * @param null|string $prettyVersion
+     *
+     * @return void
      */
     private function addExtension($name, $prettyVersion)
     {
@@ -535,7 +626,7 @@ class PlatformRepository extends ArrayRepository
             $version = $this->versionParser->normalize($prettyVersion);
         } catch (\UnexpectedValueException $e) {
             $extraDescription = ' (actual version: '.$prettyVersion.')';
-            if (preg_match('{^(\d+\.\d+\.\d+(?:\.\d+)?)}', $prettyVersion, $match)) {
+            if (Preg::isMatch('{^(\d+\.\d+\.\d+(?:\.\d+)?)}', $prettyVersion, $match)) {
                 $prettyVersion = $match[1];
             } else {
                 $prettyVersion = '0';
@@ -549,7 +640,7 @@ class PlatformRepository extends ArrayRepository
 
         if ($name === 'uuid') {
             $ext->setReplaces(array(
-                new Link('ext-uuid', 'lib-uuid', new Constraint('=', $version), Link::TYPE_REPLACE, $ext->getPrettyVersion()),
+                'lib-uuid' => new Link('ext-uuid', 'lib-uuid', new Constraint('=', $version), Link::TYPE_REPLACE, $ext->getPrettyVersion()),
             ));
         }
 
@@ -571,6 +662,8 @@ class PlatformRepository extends ArrayRepository
      * @param string|null $description
      * @param string[]    $replaces
      * @param string[]    $provides
+     *
+     * @return void
      */
     private function addLibrary($name, $prettyVersion, $description = null, array $replaces = array(), array $provides = array())
     {
@@ -587,11 +680,18 @@ class PlatformRepository extends ArrayRepository
         $lib = new CompletePackage('lib-'.$name, $version, $prettyVersion);
         $lib->setDescription($description);
 
-        $links = function ($alias) use ($name, $version, $lib) {
-            return new Link('lib-'.$name, 'lib-'.$alias, new Constraint('=', $version), Link::TYPE_REPLACE, $lib->getPrettyVersion());
-        };
-        $lib->setReplaces(array_map($links, $replaces));
-        $lib->setProvides(array_map($links, $provides));
+        $replaceLinks = array();
+        foreach ($replaces as $replace) {
+            $replace = strtolower($replace);
+            $replaceLinks[$replace] = new Link('lib-'.$name, 'lib-'.$replace, new Constraint('=', $version), Link::TYPE_REPLACE, $lib->getPrettyVersion());
+        }
+        $provideLinks = array();
+        foreach ($provides as $provide) {
+            $provide = strtolower($provide);
+            $provideLinks[$provide] = new Link('lib-'.$name, 'lib-'.$provide, new Constraint('=', $version), Link::TYPE_PROVIDE, $lib->getPrettyVersion());
+        }
+        $lib->setReplaces($replaceLinks);
+        $lib->setProvides($provideLinks);
 
         $this->addPackage($lib);
     }
@@ -610,6 +710,31 @@ class PlatformRepository extends ArrayRepository
             return $cache[$name];
         }
 
-        return $cache[$name] = (bool) preg_match(PlatformRepository::PLATFORM_PACKAGE_REGEX, $name);
+        return $cache[$name] = Preg::isMatch(PlatformRepository::PLATFORM_PACKAGE_REGEX, $name);
+    }
+
+    /**
+     * Returns the last seen config.platform.php version if defined
+     *
+     * This is a best effort attempt for internal purposes, retrieve the real
+     * packages from a PlatformRepository instance if you need a version guaranteed to
+     * be correct.
+     *
+     * @internal
+     * @return string|null
+     */
+    public static function getPlatformPhpVersion()
+    {
+        return self::$lastSeenPlatformPhp;
+    }
+
+    public function search($query, $mode = 0, $type = null)
+    {
+        // suppress vendor search as there are no vendors to match in platform packages
+        if ($mode === self::SEARCH_VENDOR) {
+            return array();
+        }
+
+        return parent::search($query, $mode, $type);
     }
 }
