@@ -17,6 +17,7 @@ use Composer\Downloader\TransportException;
 use Composer\Json\JsonFile;
 use Composer\Cache;
 use Composer\IO\IOInterface;
+use Composer\Pcre\Preg;
 use Composer\Util\GitHub;
 use Composer\Util\Http\Response;
 
@@ -25,30 +26,40 @@ use Composer\Util\Http\Response;
  */
 class GitHubDriver extends VcsDriver
 {
+    /** @var string */
     protected $owner;
+    /** @var string */
     protected $repository;
+    /** @var array<string, string> Map of tag name to identifier */
     protected $tags;
+    /** @var array<string, string> Map of branch name to identifier */
     protected $branches;
+    /** @var string */
     protected $rootIdentifier;
+    /** @var mixed[] */
     protected $repoData;
-    protected $hasIssues;
+    /** @var bool */
+    protected $hasIssues = false;
+    /** @var bool */
     protected $isPrivate = false;
+    /** @var bool */
     private $isArchived = false;
+    /** @var array<int, array{type: string, url: string}>|false|null */
     private $fundingInfo;
 
     /**
      * Git Driver
      *
-     * @var GitDriver
+     * @var ?GitDriver
      */
-    protected $gitDriver;
+    protected $gitDriver = null;
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function initialize()
     {
-        preg_match('#^(?:(?:https?|git)://([^/]+)/|git@([^:]+):/?)([^/]+)/(.+?)(?:\.git|/)?$#', $this->url, $match);
+        Preg::match('#^(?:(?:https?|git)://([^/]+)/|git@([^:]+):/?)([^/]+)/(.+?)(?:\.git|/)?$#', $this->url, $match);
         $this->owner = $match[3];
         $this->repository = $match[4];
         $this->originUrl = strtolower(!empty($match[1]) ? $match[1] : $match[2]);
@@ -67,13 +78,16 @@ class GitHubDriver extends VcsDriver
         $this->fetchRootIdentifier();
     }
 
+    /**
+     * @return string
+     */
     public function getRepositoryUrl()
     {
         return 'https://'.$this->originUrl.'/'.$this->owner.'/'.$this->repository;
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function getRootIdentifier()
     {
@@ -85,7 +99,7 @@ class GitHubDriver extends VcsDriver
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function getUrl()
     {
@@ -97,7 +111,7 @@ class GitHubDriver extends VcsDriver
     }
 
     /**
-     * {@inheritDoc}
+     * @return string
      */
     protected function getApiUrl()
     {
@@ -111,7 +125,7 @@ class GitHubDriver extends VcsDriver
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function getSource($identifier)
     {
@@ -130,7 +144,7 @@ class GitHubDriver extends VcsDriver
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function getDist($identifier)
     {
@@ -140,7 +154,7 @@ class GitHubDriver extends VcsDriver
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function getComposerInformation($identifier)
     {
@@ -182,6 +196,9 @@ class GitHubDriver extends VcsDriver
         return $this->infoCache[$identifier];
     }
 
+    /**
+     * @return array<int, array{type: string, url: string}>|false
+     */
     private function getFundingInfo()
     {
         if (null !== $this->fundingInfo) {
@@ -211,20 +228,20 @@ class GitHubDriver extends VcsDriver
 
         $result = array();
         $key = null;
-        foreach (preg_split('{\r?\n}', $funding) as $line) {
+        foreach (Preg::split('{\r?\n}', $funding) as $line) {
             $line = trim($line);
-            if (preg_match('{^(\w+)\s*:\s*(.+)$}', $line, $match)) {
-                if (preg_match('{^\[(.*)\](?:\s*#.*)?$}', $match[2], $match2)) {
-                    foreach (array_map('trim', preg_split('{[\'"]?\s*,\s*[\'"]?}', $match2[1])) as $item) {
+            if (Preg::isMatch('{^(\w+)\s*:\s*(.+)$}', $line, $match)) {
+                if (Preg::isMatch('{^\[(.*)\](?:\s*#.*)?$}', $match[2], $match2)) {
+                    foreach (array_map('trim', Preg::split('{[\'"]?\s*,\s*[\'"]?}', $match2[1])) as $item) {
                         $result[] = array('type' => $match[1], 'url' => trim($item, '"\' '));
                     }
-                } elseif (preg_match('{^([^#].*?)(\s+#.*)?$}', $match[2], $match2)) {
+                } elseif (Preg::isMatch('{^([^#].*?)(\s+#.*)?$}', $match[2], $match2)) {
                     $result[] = array('type' => $match[1], 'url' => trim($match2[1], '"\' '));
                 }
                 $key = null;
-            } elseif (preg_match('{^(\w+)\s*:\s*#\s*$}', $line, $match)) {
+            } elseif (Preg::isMatch('{^(\w+)\s*:\s*#\s*$}', $line, $match)) {
                 $key = $match[1];
-            } elseif ($key && preg_match('{^-\s*(.+)(\s+#.*)?$}', $line, $match)) {
+            } elseif ($key && Preg::isMatch('{^-\s*(.+)(\s+#.*)?$}', $line, $match)) {
                 $result[] = array('type' => $key, 'url' => trim($match[1], '"\' '));
             }
         }
@@ -265,7 +282,7 @@ class GitHubDriver extends VcsDriver
     }
 
     /**
-     * {@inheritdoc}
+     * @inheritDoc
      */
     public function getFileContent($file, $identifier)
     {
@@ -283,7 +300,7 @@ class GitHubDriver extends VcsDriver
     }
 
     /**
-     * {@inheritdoc}
+     * @inheritDoc
      */
     public function getChangeDate($identifier)
     {
@@ -298,7 +315,7 @@ class GitHubDriver extends VcsDriver
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function getTags()
     {
@@ -306,25 +323,27 @@ class GitHubDriver extends VcsDriver
             return $this->gitDriver->getTags();
         }
         if (null === $this->tags) {
-            $this->tags = array();
+            $tags = array();
             $resource = $this->getApiUrl() . '/repos/'.$this->owner.'/'.$this->repository.'/tags?per_page=100';
 
             do {
                 $response = $this->getContents($resource);
                 $tagsData = $response->decodeJson();
                 foreach ($tagsData as $tag) {
-                    $this->tags[$tag['name']] = $tag['commit']['sha'];
+                    $tags[$tag['name']] = $tag['commit']['sha'];
                 }
 
                 $resource = $this->getNextPage($response);
             } while ($resource);
+
+            $this->tags = $tags;
         }
 
         return $this->tags;
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function getBranches()
     {
@@ -332,7 +351,7 @@ class GitHubDriver extends VcsDriver
             return $this->gitDriver->getBranches();
         }
         if (null === $this->branches) {
-            $this->branches = array();
+            $branches = array();
             $resource = $this->getApiUrl() . '/repos/'.$this->owner.'/'.$this->repository.'/git/refs/heads?per_page=100';
 
             do {
@@ -341,28 +360,30 @@ class GitHubDriver extends VcsDriver
                 foreach ($branchData as $branch) {
                     $name = substr($branch['ref'], 11);
                     if ($name !== 'gh-pages') {
-                        $this->branches[$name] = $branch['object']['sha'];
+                        $branches[$name] = $branch['object']['sha'];
                     }
                 }
 
                 $resource = $this->getNextPage($response);
             } while ($resource);
+
+            $this->branches = $branches;
         }
 
         return $this->branches;
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public static function supports(IOInterface $io, Config $config, $url, $deep = false)
     {
-        if (!preg_match('#^((?:https?|git)://([^/]+)/|git@([^:]+):/?)([^/]+)/(.+?)(?:\.git|/)?$#', $url, $matches)) {
+        if (!Preg::isMatch('#^((?:https?|git)://([^/]+)/|git@([^:]+):/?)([^/]+)/(.+?)(?:\.git|/)?$#', $url, $matches)) {
             return false;
         }
 
         $originUrl = !empty($matches[2]) ? $matches[2] : $matches[3];
-        if (!in_array(strtolower(preg_replace('{^www\.}i', '', $originUrl)), $config->get('github-domains'))) {
+        if (!in_array(strtolower(Preg::replace('{^www\.}i', '', $originUrl)), $config->get('github-domains'))) {
             return false;
         }
 
@@ -378,7 +399,7 @@ class GitHubDriver extends VcsDriver
     /**
      * Gives back the loaded <github-api>/repos/<owner>/<repo> result
      *
-     * @return array|null
+     * @return mixed[]|null
      */
     public function getRepoData()
     {
@@ -402,7 +423,9 @@ class GitHubDriver extends VcsDriver
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
+     *
+     * @param bool $fetchingRepoData
      */
     protected function getContents($url, $fetchingRepoData = false)
     {
@@ -424,9 +447,9 @@ class GitHubDriver extends VcsDriver
                     }
 
                     if (!$this->io->isInteractive()) {
-                        if ($this->attemptCloneFallback()) {
-                            return new Response(array('url' => 'dummy'), 200, array(), 'null');
-                        }
+                        $this->attemptCloneFallback();
+
+                        return new Response(array('url' => 'dummy'), 200, array(), 'null');
                     }
 
                     $scopesIssued = array();
@@ -454,12 +477,12 @@ class GitHubDriver extends VcsDriver
                     }
 
                     if (!$this->io->isInteractive() && $fetchingRepoData) {
-                        if ($this->attemptCloneFallback()) {
-                            return new Response(array('url' => 'dummy'), 200, array(), 'null');
-                        }
+                        $this->attemptCloneFallback();
+
+                        return new Response(array('url' => 'dummy'), 200, array(), 'null');
                     }
 
-                    $rateLimited = $gitHubUtil->isRateLimited($e->getHeaders());
+                    $rateLimited = $gitHubUtil->isRateLimited((array) $e->getHeaders());
 
                     if (!$this->io->hasAuthentication($this->originUrl)) {
                         if (!$this->io->isInteractive()) {
@@ -492,6 +515,7 @@ class GitHubDriver extends VcsDriver
     /**
      * Fetch root identifier from GitHub
      *
+     * @return void
      * @throws TransportException
      */
     protected function fetchRootIdentifier()
@@ -530,6 +554,12 @@ class GitHubDriver extends VcsDriver
         $this->isArchived = !empty($this->repoData['archived']);
     }
 
+    /**
+     * @phpstan-impure
+     *
+     * @return true
+     * @throws \RuntimeException
+     */
     protected function attemptCloneFallback()
     {
         $this->isPrivate = true;
@@ -550,6 +580,11 @@ class GitHubDriver extends VcsDriver
         }
     }
 
+    /**
+     * @param string $url
+     *
+     * @return void
+     */
     protected function setupGitDriver($url)
     {
         $this->gitDriver = new GitDriver(
@@ -562,15 +597,23 @@ class GitHubDriver extends VcsDriver
         $this->gitDriver->initialize();
     }
 
+    /**
+     * @return string|null
+     */
     protected function getNextPage(Response $response)
     {
         $header = $response->getHeader('link');
+        if (!$header) {
+            return null;
+        }
 
         $links = explode(',', $header);
         foreach ($links as $link) {
-            if (preg_match('{<(.+?)>; *rel="next"}', $link, $match)) {
+            if (Preg::isMatch('{<(.+?)>; *rel="next"}', $link, $match)) {
                 return $match[1];
             }
         }
+
+        return null;
     }
 }
