@@ -13,6 +13,7 @@
 namespace Composer\Util;
 
 use Composer\IO\IOInterface;
+use Composer\Pcre\Preg;
 use Symfony\Component\Process\Process;
 
 /**
@@ -26,7 +27,7 @@ class Perforce
     protected $path;
     /** @var ?string */
     protected $p4Depot;
-    /** @var string */
+    /** @var ?string */
     protected $p4Client;
     /** @var ?string */
     protected $p4User;
@@ -34,7 +35,7 @@ class Perforce
     protected $p4Password;
     /** @var string */
     protected $p4Port;
-    /** @var string */
+    /** @var ?string */
     protected $p4Stream;
     /** @var string */
     protected $p4ClientSpec;
@@ -54,7 +55,7 @@ class Perforce
     /** @var IOInterface */
     protected $io;
 
-    /** @var Filesystem */
+    /** @var ?Filesystem */
     protected $filesystem;
 
     /**
@@ -634,7 +635,7 @@ class Perforce
             foreach ($resArray as $line) {
                 $resBits = explode(' ', $line);
                 if (count($resBits) > 4) {
-                    $branch = preg_replace('/[^A-Za-z0-9 ]/', '', $resBits[4]);
+                    $branch = Preg::replace('/[^A-Za-z0-9 ]/', '', $resBits[4]);
                     $possibleBranches[$branch] = $resBits[1];
                 }
             }
@@ -743,7 +744,7 @@ class Perforce
      */
     public function getFilesystem()
     {
-        if (empty($this->filesystem)) {
+        if (null === $this->filesystem) {
             $this->filesystem = new Filesystem($this->process);
         }
 
