@@ -15,7 +15,7 @@ namespace Composer\Test\Package\Archiver;
 use Composer\IO\NullIO;
 use Composer\Factory;
 use Composer\Package\Archiver\ArchiveManager;
-use Composer\Package\PackageInterface;
+use Composer\Package\CompletePackage;
 use Composer\Util\Loop;
 use Composer\Test\Mock\FactoryMock;
 use Composer\Util\ProcessExecutor;
@@ -27,6 +27,9 @@ class ArchiveManagerTest extends ArchiverTest
      */
     protected $manager;
 
+    /**
+     * @var string
+     */
     protected $targetDir;
 
     public function setUp()
@@ -95,7 +98,13 @@ class ArchiveManagerTest extends ArchiverTest
         unlink($target);
     }
 
-    protected function getTargetName(PackageInterface $package, $format, $fileName = null)
+    /**
+     * @param string      $format
+     * @param string|null $fileName
+     *
+     * @return string
+     */
+    protected function getTargetName(CompletePackage $package, $format, $fileName = null)
     {
         if (null === $fileName) {
             $packageName = $this->manager->getPackageFilename($package);
@@ -108,6 +117,8 @@ class ArchiveManagerTest extends ArchiverTest
 
     /**
      * Create local git repository to run tests against!
+     *
+     * @return void
      */
     protected function setupGitRepo()
     {
