@@ -39,6 +39,9 @@ class ZipArchiverTest extends ArchiverTest
         );
     }
 
+    /**
+     * @param array<string, string> $files
+     */
     public function testZipArchive(array $files = array())
     {
         if (!class_exists('ZipArchive')) {
@@ -47,14 +50,14 @@ class ZipArchiverTest extends ArchiverTest
 
         if (empty($files)) {
             $files = array(
-                'file.txt' => NULL,
-                'foo/bar/baz' => NULL,
-                'x/baz' => NULL,
-                'x/includeme' => NULL,
+                'file.txt' => null,
+                'foo/bar/baz' => null,
+                'x/baz' => null,
+                'x/includeme' => null,
             );
 
             if (!Platform::isWindows()) {
-                $files['foo' . getcwd() . '/file.txt'] = NULL;
+                $files['foo' . getcwd() . '/file.txt'] = null;
             }
         }
         // Set up repository
@@ -79,14 +82,17 @@ class ZipArchiverTest extends ArchiverTest
 
     /**
      * Create a local dummy repository to run tests against!
-     * @param array $files
+     *
+     * @param array<string, string|null> $files
+     *
+     * @return void
      */
     protected function setupDummyRepo(array &$files)
     {
         $currentWorkDir = getcwd();
         chdir($this->testDir);
         foreach ($files as $path => $content) {
-            if ($files[$path] === NULL) {
+            if ($files[$path] === null) {
                 $files[$path] = 'content';
             }
             $this->writeFile($path, $files[$path], $currentWorkDir);
@@ -95,6 +101,13 @@ class ZipArchiverTest extends ArchiverTest
         chdir($currentWorkDir);
     }
 
+    /**
+     * @param string $path
+     * @param string $content
+     * @param string $currentWorkDir
+     *
+     * @return void
+     */
     protected function writeFile($path, $content, $currentWorkDir)
     {
         if (!file_exists(dirname($path))) {
