@@ -15,6 +15,7 @@ namespace Composer\Test\Plugin;
 use Composer\Composer;
 use Composer\Config;
 use Composer\Installer\PluginInstaller;
+use Composer\Package\CompleteAliasPackage;
 use Composer\Package\CompletePackage;
 use Composer\Package\Loader\JsonLoader;
 use Composer\Package\Loader\ArrayLoader;
@@ -44,7 +45,7 @@ class PluginInstallerTest extends TestCase
     protected $autoloadGenerator;
 
     /**
-     * @var CompletePackage[]
+     * @var array<CompletePackage|CompleteAliasPackage>
      */
     protected $packages;
 
@@ -114,16 +115,17 @@ class PluginInstallerTest extends TestCase
         $this->composer->setEventDispatcher(new EventDispatcher($this->composer, $this->io));
         $this->composer->setPackage(new RootPackage('dummy/root', '1.0.0.0', '1.0.0'));
 
-        $this->pm = new PluginManager($this->io, $this->composer);
-        $this->composer->setPluginManager($this->pm);
-
         $config->merge(array(
             'config' => array(
                 'vendor-dir' => $this->directory.'/Fixtures/',
                 'home' => $this->directory.'/Fixtures',
                 'bin-dir' => $this->directory.'/Fixtures/bin',
+                'allow-plugins' => true,
             ),
         ));
+
+        $this->pm = new PluginManager($this->io, $this->composer);
+        $this->composer->setPluginManager($this->pm);
     }
 
     protected function tearDown()
@@ -145,7 +147,32 @@ class PluginInstallerTest extends TestCase
 
         $plugins = $this->pm->getPlugins();
         $this->assertEquals('installer-v1', $plugins[0]->version);  // @phpstan-ignore-line
-        $this->assertEquals('activate v1'.PHP_EOL, $this->io->getOutput());
+        $this->assertEquals(
+            'activate v1'.PHP_EOL,
+            $this->io->getOutput()
+        );
+    }
+
+    public function testInstallPluginWithRootPackageHavingFilesAutoload()
+    {
+        $this->repository
+            ->expects($this->any())
+            ->method('getPackages')
+            ->will($this->returnValue(array()));
+        $installer = new PluginInstaller($this->io, $this->composer);
+        $this->pm->loadInstalledPlugins();
+
+        $this->autoloadGenerator->setDevMode(true);
+        $this->composer->getPackage()->setAutoload(array('files' => array(__DIR__ . '/Fixtures/files_autoload_which_should_not_run.php')));
+        $this->composer->getPackage()->setDevAutoload(array('files' => array(__DIR__ . '/Fixtures/files_autoload_which_should_not_run.php')));
+        $installer->install($this->repository, $this->packages[0]);
+
+        $plugins = $this->pm->getPlugins();
+        $this->assertEquals(
+            'activate v1'.PHP_EOL,
+            $this->io->getOutput()
+        );
+        $this->assertEquals('installer-v1', $plugins[0]->version);  // @phpstan-ignore-line
     }
 
     public function testInstallMultiplePlugins()
@@ -248,7 +275,7 @@ class PluginInstallerTest extends TestCase
 
     /**
      * @param string            $newPluginApiVersion
-     * @param CompletePackage[] $plugins
+     * @param array<CompletePackage|CompleteAliasPackage> $plugins
      *
      * @return void
      */
