@@ -14,7 +14,6 @@ namespace Composer\Test\Util;
 
 use Composer\Config;
 use Composer\IO\NullIO;
-use Composer\Util\Platform;
 use Composer\Util\Svn;
 use Composer\Test\TestCase;
 
@@ -37,11 +36,6 @@ class SvnTest extends TestCase
         $this->assertEquals($expect, $reflMethod->invoke($svn));
     }
 
-    /**
-     * Provide some examples for {@self::testCredentials()}.
-     *
-     * @return array
-     */
     public function urlProvider()
     {
         return array(
@@ -130,9 +124,4 @@ class SvnTest extends TestCase
 
         $this->assertEquals($this->getCmd(" --no-auth-cache --username 'foo' --password 'bar' "), $reflMethod->invoke($svn));
     }
-
-    private function getCmd($cmd)
-    {
-        return Platform::isWindows() ? strtr($cmd, "'", '"') : $cmd;
-    }
 }
